@@ -1,7 +1,9 @@
 import inspect
 import itertools
-from collections import Iterable
+from collections.abc import Iterable
 import re
+import warnings
+from typing import Callable, Dict
 
 import numpy as np
 import scipy
@@ -10,10 +12,12 @@ from Orange.data import Table, Storage, Instance, Value
 from Orange.data.filter import HasClass
 from Orange.data.table import DomainTransformationError
 from Orange.data.util import one_hot
+from Orange.misc.environ import cache_dir
 from Orange.misc.wrapper_meta import WrapperMeta
 from Orange.preprocess import Continuize, RemoveNaNColumns, SklImpute, Normalize
 from Orange.statistics.util import all_nan
-from Orange.util import Reprable
+from Orange.util import Reprable, OrangeDeprecationWarning, wrap_callback, \
+    dummy_callback
 
 __all__ = ["Learner", "Model", "SklLearner", "SklModel",
            "ReprableWithPreprocessors"]
@@ -101,7 +105,7 @@ class Learner(ReprableWithPreprocessors):
         X, Y, W = data.X, data.Y, data.W if data.has_weights() else None
         return self.fit(X, Y, W)
 
-    def __call__(self, data):
+    def __call__(self, data, progress_callback=None):
         if not self.check_learner_adequacy(data.domain):
             raise ValueError(self.learner_adequacy_err_msg)
 
@@ -110,12 +114,26 @@ class Learner(ReprableWithPreprocessors):
         if isinstance(data, Instance):
             data = Table(data.domain, [data])
         origdata = data
-        data = self.preprocess(data)
+
+        if progress_callback is None:
+            progress_callback = dummy_callback
+        progress_callback(0, "Preprocessing...")
+        try:
+            cb = wrap_callback(progress_callback, end=0.1)
+            data = self.preprocess(data, progress_callback=cb)
+        except TypeError:
+            data = self.preprocess(data)
+            warnings.warn("A keyword argument 'progress_callback' has been "
+                          "added to the preprocess() signature. Implementing "
+                          "the method without the argument is deprecated and "
+                          "will result in an error in the future.",
+                          OrangeDeprecationWarning)
 
         if len(data.domain.class_vars) > 1 and not self.supports_multiclass:
             raise TypeError("%s doesn't support multiple class variables" %
                             self.__class__.__name__)
 
+        progress_callback(0.1, "Fitting...")
         model = self._fit_model(data)
         model.used_vals = [np.unique(y).astype(int) for y in data.Y[:, None].T]
         model.domain = data.domain
@@ -123,6 +141,7 @@ class Learner(ReprableWithPreprocessors):
         model.name = self.name
         model.original_domain = origdomain
         model.original_data = origdata
+        progress_callback(1)
         return model
 
     def _fit_model(self, data):
@@ -132,10 +151,15 @@ class Learner(ReprableWithPreprocessors):
             X, Y, W = data.X, data.Y, data.W if data.has_weights() else None
             return self.fit(X, Y, W)
 
-    def preprocess(self, data):
+    def preprocess(self, data, progress_callback=None):
         """Apply the `preprocessors` to the data"""
-        for pp in self.active_preprocessors:
+        if progress_callback is None:
+            progress_callback = dummy_callback
+        n_pps = len(list(self.active_preprocessors))
+        for i, pp in enumerate(self.active_preprocessors):
+            progress_callback(i / n_pps)
             data = pp(data)
+        progress_callback(1)
         return data
 
     @property
@@ -145,7 +169,7 @@ class Learner(ReprableWithPreprocessors):
                 self.preprocessors is not type(self).preprocessors):
             yield from type(self).preprocessors
 
-    def check_learner_adequacy(self, domain):
+    def check_learner_adequacy(self, _):
         return True
 
     @property
@@ -183,15 +207,12 @@ class Model(Reprable):
     ValueProbs = 2
 
     def __init__(self, domain=None, original_domain=None):
-        if isinstance(self, Learner):
-            domain = None
-        elif domain is None:
-            raise ValueError("unspecified domain")
         self.domain = domain
         if original_domain is not None:
             self.original_domain = original_domain
         else:
             self.original_domain = domain
+        self.used_vals = None
 
     def predict(self, X):
         if type(self).predict_storage is Model.predict_storage:
@@ -298,12 +319,61 @@ class Model(Reprable):
         new_probs = new_probs / tots[:, None]
         return new_probs
 
+    def data_to_model_domain(
+            self, data: Table, progress_callback: Callable = dummy_callback
+    ) -> Table:
+        """
+        Transforms data to the model domain if possible.
+
+        Parameters
+        ----------
+        data
+            Data to be transformed to the model domain
+        progress_callback
+            Callback - callable - to report the progress
+
+        Returns
+        -------
+        Transformed data table
+
+        Raises
+        ------
+        DomainTransformationError
+            Error indicates that transformation is not possible since domains
+            are not compatible
+        """
+        if data.domain == self.domain:
+            return data
+
+        progress_callback(0)
+        if self.original_domain.attributes != data.domain.attributes \
+                and data.X.size \
+                and not all_nan(data.X):
+            progress_callback(0.5)
+            new_data = data.transform(self.original_domain)
+            if all_nan(new_data.X):
+                raise DomainTransformationError(
+                    "domain transformation produced no defined values")
+            progress_callback(0.75)
+            data = new_data.transform(self.domain)
+            progress_callback(1)
+            return data
+
+        progress_callback(0.5)
+        data = data.transform(self.domain)
+        progress_callback(1)
+        return data
+
     def __call__(self, data, ret=Value):
         multitarget = len(self.domain.class_vars) > 1
 
         def one_hot_probs(value):
             if not multitarget:
-                return one_hot(value)
+                return one_hot(
+                    value,
+                    dim=len(self.domain.class_var.values)
+                    if self.domain is not None else None
+                )
 
             max_card = max(len(c.values) for c in self.domain.class_vars)
             probs = np.zeros(value.shape + (max_card,), float)
@@ -311,24 +381,33 @@ class Model(Reprable):
                 probs[:, i, :] = one_hot(value[:, i])
             return probs
 
+        def extend_probabilities(probs):
+            """
+            Since SklModels and models implementing `fit` and not `fit_storage`
+            do not guarantee correct prediction dimensionality, extend
+            dimensionality of probabilities when it does not match the number
+            of values in the domain.
+            """
+            class_vars = self.domain.class_vars
+            max_values = max(len(cv.values) for cv in class_vars)
+            if max_values == probs.shape[-1]:
+                return probs
+
+            if not self.supports_multiclass:
+                probs = probs[:, np.newaxis, :]
+
+            probs_ext = np.zeros((len(probs), len(class_vars), max_values))
+            for c, used_vals in enumerate(self.used_vals):
+                for i, cv in enumerate(used_vals):
+                    probs_ext[:, c, cv] = probs[:, c, i]
+
+            if not self.supports_multiclass:
+                probs_ext = probs_ext[:, 0, :]
+            return probs_ext
+
         def fix_dim(x):
             return x[0] if one_d else x
 
-        def data_to_model_domain():
-            if data.domain == self.domain:
-                return data
-
-            if self.original_domain.attributes != data.domain.attributes \
-                    and data.X.size \
-                    and not all_nan(data.X):
-                new_data = data.transform(self.original_domain)
-                if all_nan(new_data.X):
-                    raise DomainTransformationError(
-                        "domain transformation produced no defined values")
-                return new_data.transform(self.domain)
-
-            return data.transform(self.domain)
-
         if not 0 <= ret <= 2:
             raise ValueError("invalid value of argument 'ret'")
         if ret > 0 and any(v.is_continuous for v in self.domain.class_vars):
@@ -346,6 +425,10 @@ class Model(Reprable):
         else:
             one_d = False
 
+        # if sparse convert to csr_matrix
+        if scipy.sparse.issparse(data):
+            data = data.tocsr()
+
         # Call the predictor
         backmappers = None
         n_values = []
@@ -353,7 +436,7 @@ class Model(Reprable):
             prediction = self.predict(data)
         elif isinstance(data, Table):
             backmappers, n_values = self.get_backmappers(data)
-            data = data_to_model_domain()
+            data = self.data_to_model_domain(data)
             prediction = self.predict_storage(data)
         elif isinstance(data, (list, tuple)):
             data = Table.from_list(self.original_domain, data)
@@ -378,6 +461,7 @@ class Model(Reprable):
         if probs is None and (ret != Model.Value or backmappers is not None):
             probs = one_hot_probs(value)
         if probs is not None:
+            probs = extend_probabilities(probs)
             probs = self.backmap_probs(probs, n_values, backmappers)
         if ret != Model.Probs:
             if value is None:
@@ -459,17 +543,20 @@ class SklLearner(Learner, metaclass=WrapperMeta):
     def _get_sklparams(self, values):
         skllearner = self.__wraps__
         if skllearner is not None:
-            spec = inspect.getargs(skllearner.__init__.__code__)
+            spec = list(
+                inspect.signature(skllearner.__init__).parameters.keys()
+            )
             # first argument is 'self'
-            assert spec.args[0] == "self"
-            params = {name: values[name] for name in spec.args[1:]
-                      if name in values}
+            assert spec[0] == "self"
+            params = {
+                name: values[name] for name in spec[1:] if name in values
+            }
         else:
             raise TypeError("Wrapper does not define '__wraps__'")
         return params
 
-    def preprocess(self, data):
-        data = super().preprocess(data)
+    def preprocess(self, data, progress_callback=None):
+        data = super().preprocess(data, progress_callback)
 
         if any(v.is_discrete and len(v.values) > 2
                for v in data.domain.attributes):
@@ -478,12 +565,13 @@ class SklLearner(Learner, metaclass=WrapperMeta):
 
         return data
 
-    def __call__(self, data):
-        m = super().__call__(data)
+    def __call__(self, data, progress_callback=None):
+        m = super().__call__(data, progress_callback)
         m.params = self.params
         return m
 
     def _initialize_wrapped(self):
+        # pylint: disable=not-callable
         return self.__wraps__(**self.params)
 
     def fit(self, X, Y, W=None):
@@ -533,6 +621,8 @@ class RandomForestModel(Model):
 class KNNBase:
     """Base class for KNN (classification and regression) learners
     """
+
+    # pylint: disable=unused-argument
     def __init__(self, n_neighbors=5, metric="euclidean", weights="uniform",
                  algorithm='auto', metric_params=None,
                  preprocessors=None):
@@ -551,6 +641,7 @@ class NNBase:
     """
     preprocessors = SklLearner.preprocessors + [Normalize()]
 
+    # pylint: disable=unused-argument,too-many-arguments
     def __init__(self, hidden_layer_sizes=(100,), activation='relu',
                  solver='adam', alpha=0.0001, batch_size='auto',
                  learning_rate='constant', learning_rate_init=0.001,
@@ -561,3 +652,209 @@ class NNBase:
                  epsilon=1e-08, preprocessors=None):
         super().__init__(preprocessors=preprocessors)
         self.params = vars()
+
+
+class CatGBModel(Model, metaclass=WrapperMeta):
+    def __init__(self, cat_model, cat_features, domain):
+        super().__init__(domain)
+        self.cat_model = cat_model
+        self.cat_features = cat_features
+
+    def predict(self, X):
+        if self.cat_features:
+            X = X.astype(str)
+        value = self.cat_model.predict(X).flatten()
+        if hasattr(self.cat_model, "predict_proba"):
+            probs = self.cat_model.predict_proba(X)
+            return value, probs
+        return value
+
+    def __repr__(self):
+        # Params represented as a comment because not passed into constructor
+        return super().__repr__() + '  # params=' + repr(self.params)
+
+
+class CatGBBaseLearner(Learner, metaclass=WrapperMeta):
+    """
+    ${skldoc}
+    Additional Orange parameters
+
+    preprocessors : list, optional
+        An ordered list of preprocessors applied to data before
+        training or testing.
+        Defaults to
+        `[RemoveNaNClasses(), RemoveNaNColumns()]`
+    """
+    supports_weights = True
+    __wraps__ = None
+    __returns__ = CatGBModel
+    _params = {}
+    preprocessors = default_preprocessors = [
+        HasClass(),
+        RemoveNaNColumns(),
+    ]
+
+    # pylint: disable=unused-argument,too-many-arguments,too-many-locals
+    def __init__(self,
+                 iterations=None,
+                 learning_rate=None,
+                 depth=None,
+                 l2_leaf_reg=None,
+                 model_size_reg=None,
+                 rsm=None,
+                 loss_function=None,
+                 border_count=None,
+                 feature_border_type=None,
+                 per_float_feature_quantization=None,
+                 input_borders=None,
+                 output_borders=None,
+                 fold_permutation_block=None,
+                 od_pval=None,
+                 od_wait=None,
+                 od_type=None,
+                 nan_mode=None,
+                 counter_calc_method=None,
+                 leaf_estimation_iterations=None,
+                 leaf_estimation_method=None,
+                 thread_count=None,
+                 random_seed=None,
+                 use_best_model=None,
+                 verbose=False,
+                 logging_level=None,
+                 metric_period=None,
+                 ctr_leaf_count_limit=None,
+                 store_all_simple_ctr=None,
+                 max_ctr_complexity=None,
+                 has_time=None,
+                 allow_const_label=None,
+                 classes_count=None,
+                 class_weights=None,
+                 one_hot_max_size=None,
+                 random_strength=None,
+                 name=None,
+                 ignored_features=None,
+                 train_dir=cache_dir(),
+                 custom_loss=None,
+                 custom_metric=None,
+                 eval_metric=None,
+                 bagging_temperature=None,
+                 save_snapshot=None,
+                 snapshot_file=None,
+                 snapshot_interval=None,
+                 fold_len_multiplier=None,
+                 used_ram_limit=None,
+                 gpu_ram_part=None,
+                 allow_writing_files=False,
+                 final_ctr_computation_mode=None,
+                 approx_on_full_history=None,
+                 boosting_type=None,
+                 simple_ctr=None,
+                 combinations_ctr=None,
+                 per_feature_ctr=None,
+                 task_type=None,
+                 device_config=None,
+                 devices=None,
+                 bootstrap_type=None,
+                 subsample=None,
+                 sampling_unit=None,
+                 dev_score_calc_obj_block_size=None,
+                 max_depth=None,
+                 n_estimators=None,
+                 num_boost_round=None,
+                 num_trees=None,
+                 colsample_bylevel=None,
+                 random_state=None,
+                 reg_lambda=None,
+                 objective=None,
+                 eta=None,
+                 max_bin=None,
+                 scale_pos_weight=None,
+                 gpu_cat_features_storage=None,
+                 data_partition=None,
+                 metadata=None,
+                 early_stopping_rounds=None,
+                 cat_features=None,
+                 grow_policy=None,
+                 min_data_in_leaf=None,
+                 min_child_samples=None,
+                 max_leaves=None,
+                 num_leaves=None,
+                 score_function=None,
+                 leaf_estimation_backtracking=None,
+                 ctr_history_unit=None,
+                 monotone_constraints=None,
+                 feature_weights=None,
+                 penalties_coefficient=None,
+                 first_feature_use_penalties=None,
+                 model_shrink_rate=None,
+                 model_shrink_mode=None,
+                 langevin=None,
+                 diffusion_temperature=None,
+                 posterior_sampling=None,
+                 boost_from_average=None,
+                 text_features=None,
+                 tokenizers=None,
+                 dictionaries=None,
+                 feature_calcers=None,
+                 text_processing=None,
+                 preprocessors=None):
+        super().__init__(preprocessors=preprocessors)
+        self.params = vars()
+
+    @property
+    def params(self):
+        return self._params
+
+    @params.setter
+    def params(self, value):
+        self._params = self._get_wrapper_params(value)
+
+    def _get_wrapper_params(self, values):
+        spec = list(inspect.signature(
+            self.__wraps__.__init__).parameters.keys())
+        return {name: values[name] for name in spec[1:] if name in values}
+
+    def __call__(self, data, progress_callback=None):
+        m = super().__call__(data, progress_callback)
+        m.params = self.params
+        return m
+
+    def fit_storage(self, data: Table):
+        domain, X, Y, W = data.domain, data.X, data.Y.reshape(-1), None
+        if self.supports_weights and data.has_weights():
+            W = data.W.reshape(-1)
+        # pylint: disable=not-callable
+        clf = self.__wraps__(**self.params)
+        cat_features = [i for i, attr in enumerate(domain.attributes)
+                        if attr.is_discrete]
+        if cat_features:
+            X = X.astype(str)
+        cat_model = clf.fit(X, Y, cat_features=cat_features, sample_weight=W)
+        return self.__returns__(cat_model, cat_features, domain)
+
+    def __getattr__(self, item):
+        try:
+            return self.params[item]
+        except (KeyError, AttributeError):
+            raise AttributeError(item) from None
+
+    def __dir__(self):
+        dd = super().__dir__()
+        return list(sorted(set(dd) | set(self.params.keys())))
+
+
+class XGBBase(SklLearner):
+    """Base class for xgboost (classification and regression) learners """
+    preprocessors = default_preprocessors = [
+        HasClass(),
+        Continuize(),
+        RemoveNaNColumns(),
+    ]
+
+    def __init__(self, preprocessors=None, **kwargs):
+        super().__init__(preprocessors=preprocessors)
+        self.params = kwargs
+
+    @SklLearner.params.setter
+    def params(self, values: Dict):
+        self._params = values
