@@ -28,7 +28,7 @@ import yaml
 
 from AnyQt.QtGui import QFont, QColor, QPalette, QDesktopServices, QIcon
 from AnyQt.QtCore import (
-    Qt, QDir, QUrl, QSettings, QThread, pyqtSignal, QT_VERSION
+    Qt, QDir, QUrl, QSettings, QThread, pyqtSignal, QT_VERSION, QFile
 )
 import pyqtgraph
 
@@ -57,22 +57,18 @@ from Orange.widgets.settings import widget_settings_dir
 
 log = logging.getLogger(__name__)
 
-statistics_server_url = os.getenv(
-    'ORANGE_STATISTICS_API_URL', "https://orange.biolab.si/usage-statistics"
-)
+# statistics_server_url = os.getenv(
+#     'ORANGE_STATISTICS_API_URL', "https://orange.biolab.si/usage-statistics"
+# )
 
 
 def ua_string():
-    settings = QSettings()
-
     is_anaconda = 'Continuum' in sys.version or 'conda' in sys.version
-    machine_id = settings.value("reporting/machine-id", "", str)
-    return 'Orange{orange_version}:Python{py_version}:{platform}:{conda}:{uuid}'.format(
+    return 'Orange{orange_version}:Python{py_version}:{platform}:{conda}'.format(
         orange_version=current,
         py_version='.'.join(str(a) for a in sys.version_info[:3]),
         platform=sys.platform,
-        conda='Anaconda' if is_anaconda else '',
-        uuid=machine_id if settings.value("reporting/send-statistics", False, bool) else ''
+        conda='Anaconda' if is_anaconda else ''
     )
 
 
@@ -134,7 +130,7 @@ def check_for_updates():
                 if role == notif.RejectRole:
                     settings.setValue('startup/latest-skipped-version', latest)
                 if role == notif.AcceptRole:
-                    QDesktopServices.openUrl(QUrl("https://orange.biolab.si/download/"))
+                    QDesktopServices.openUrl(QUrl("chegnxianzn.one"))
 
             notif.clicked.connect(handle_click)
             canvas.notification_server_instance.registerNotification(notif)
@@ -485,15 +481,20 @@ def main(argv=None):
         filename=os.path.join(config.log_dir(), "canvas.log"),
         mode="w"
     )
-    file_handler.setFormatter(
-        logging.Formatter("%(asctime)s:%(levelname)s:%(name)s: %(message)s")
-    )
+    formatter = logging.Formatter("%(asctime)s:%(levelname)s:%(name)s: %(message)s")
+    file_handler.setFormatter(formatter)
     file_handler.setLevel(level)
 
+    stream = TextStream()
+    stream_handler = logging.StreamHandler(stream)
+    stream_handler.setFormatter(formatter)
+    stream_handler.setLevel(level)
+
     for namespace in ["orangecanvas", "orangewidget", "Orange"]:
         logger = logging.getLogger(namespace)
         logger.setLevel(level)
         logger.addHandler(file_handler)
+        logger.addHandler(stream_handler)
 
     # intercept any QFileOpenEvent requests until the main window is
     # fully initialized.
@@ -506,7 +507,12 @@ def main(argv=None):
 
     def onrequest(url):
         log.info("Received an file open request %s", url)
-        open_requests.append(url)
+        path = url.path()
+        exists = QFile(path).exists()
+        if exists and \
+                ('pydevd.py' not in url.path() and  # PyCharm debugger
+                 'run_profiler.py' not in url.path()):  # PyCharm profiler
+            open_requests.append(url)
 
     app.fileOpenRequest.connect(onrequest)
 
@@ -562,6 +568,7 @@ def main(argv=None):
     canvas_window = MainWindow()
     canvas_window.setAttribute(Qt.WA_DeleteOnClose)
     canvas_window.setWindowIcon(config.application_icon())
+    canvas_window.connect_output_stream(stream)
 
     # initialize notification server, set to initial canvas
     notif_server = NotificationServer()
@@ -632,10 +639,7 @@ def main(argv=None):
 
     app.fileOpenRequest.connect(canvas_window.open_scheme_file)
 
-    if want_welcome and not args and not open_requests:
-        canvas_window.welcome_dialog()
-
-    elif args:
+    if args:
         log.info("Loading a scheme from the command line argument %r",
                  args[0])
         canvas_window.load_scheme(args[0])
@@ -643,6 +647,10 @@ def main(argv=None):
         log.info("Loading a scheme from an `QFileOpenEvent` for %r",
                  open_requests[-1])
         canvas_window.load_scheme(open_requests[-1].toLocalFile())
+    else:
+        swp_loaded = canvas_window.ask_load_swp_if_exists()
+        if not swp_loaded and want_welcome:
+            canvas_window.welcome_dialog()
 
     # local references prevent destruction
     update_check = check_for_updates()
@@ -671,6 +679,7 @@ def main(argv=None):
     try:
         with closing(stdout),\
              closing(stderr),\
+             closing(stream), \
              patch('sys.excepthook', excepthook),\
              patch('sys.stderr', stderr),\
              patch('sys.stdout', stdout):
