@@ -4,7 +4,7 @@ import threading
 import warnings
 import weakref
 import zlib
-from collections import Iterable, Sequence, Sized
+from collections.abc import Iterable, Sequence, Sized
 from functools import reduce
 from itertools import chain
 from numbers import Real, Integral
@@ -15,14 +15,14 @@ import numpy as np
 from Orange.misc.collections import frozendict
 from Orange.util import OrangeDeprecationWarning
 from scipy import sparse as sp
-from scipy.sparse import issparse
+from scipy.sparse import issparse, csc_matrix
 
 import Orange.data  # import for io.py
 from Orange.data import (
     _contingency, _valuecount,
     Domain, Variable, Storage, StringVariable, Unknown, Value, Instance,
-    ContinuousVariable, DiscreteVariable, MISSING_VALUES
-)
+    ContinuousVariable, DiscreteVariable, MISSING_VALUES,
+    DomainConversion)
 from Orange.data.util import SharedComputeValue, \
     assure_array_dense, assure_array_sparse, \
     assure_column_dense, assure_column_sparse, get_unique_names_duplicates
@@ -349,47 +349,80 @@ class Table(Sequence, Storage):
                     source._Y, row_indices,
                     [x - n_src_attrs for x in src_cols]))
 
-            # initialize final array & set `match_density` for columns
-            if is_sparse:
-                a = sp.dok_matrix((n_rows, len(src_cols)), dtype=dtype)
-                match_density = assure_column_sparse
-            else:
-                a = np.empty((n_rows, len(src_cols)), dtype=dtype)
-                match_density = assure_column_dense
+            # initialize arrays & set `match_density` for columns
+            # F-order enables faster writing to the array while accessing and
+            # matrix operations work with same speed (e.g. dot)
+            a = None if is_sparse else np.zeros(
+                (n_rows, len(src_cols)), order="F", dtype=dtype)
+            data = []
+            sp_col = []
+            sp_row = []
+            match_density = (
+                assure_column_sparse if is_sparse else assure_column_dense
+            )
+
+            # converting to csc before instead of each column is faster
+            # do not convert if not required
+            if any([isinstance(x, int) for x in src_cols]):
+                X = csc_matrix(source.X) if is_sparse else source.X
+                Y = csc_matrix(source._Y) if is_sparse else source._Y
 
             shared_cache = _thread_local.conversion_cache
             for i, col in enumerate(src_cols):
                 if col is None:
-                    a[:, i] = variables[i].Unknown
+                    col_array = match_density(
+                        np.full((n_rows, 1), variables[i].Unknown)
+                    )
                 elif not isinstance(col, Integral):
                     if isinstance(col, SharedComputeValue):
-                        shared, weakrefs = shared_cache.get((id(col.compute_shared), id(source)),
-                                                            (None, None))
+                        shared, weakrefs = shared_cache.get(
+                            (id(col.compute_shared), id(source)),
+                            (None, None)
+                        )
                         if shared is None or not valid_refs(weakrefs):
                             shared, _ = shared_cache[(id(col.compute_shared), id(source))] = \
                                 col.compute_shared(source), \
                                 (weakref.ref(col.compute_shared), weakref.ref(source))
 
                         if row_indices is not ...:
-                            a[:, i] = match_density(
+                            col_array = match_density(
                                 col(source, shared_data=shared)[row_indices])
                         else:
-                            a[:, i] = match_density(
+                            col_array = match_density(
                                 col(source, shared_data=shared))
                     else:
                         if row_indices is not ...:
-                            a[:, i] = match_density(col(source)[row_indices])
+                            col_array = match_density(col(source)[row_indices])
                         else:
-                            a[:, i] = match_density(col(source))
+                            col_array = match_density(col(source))
                 elif col < 0:
-                    a[:, i] = match_density(source.metas[row_indices, -1 - col])
+                    col_array = match_density(
+                        source.metas[row_indices, -1 - col]
+                    )
                 elif col < n_src_attrs:
-                    a[:, i] = match_density(source.X[row_indices, col])
+                    col_array = match_density(X[row_indices, col])
                 else:
-                    a[:, i] = match_density(
-                        source._Y[row_indices, col - n_src_attrs])
+                    col_array = match_density(
+                        Y[row_indices, col - n_src_attrs]
+                    )
+
+                if is_sparse:
+                    # col_array should be coo matrix
+                    data.append(col_array.data)
+                    sp_col.append(np.full(len(col_array.data), i))
+                    sp_row.append(col_array.indices)  # row indices should be same
+                else:
+                    a[:, i] = col_array
 
             if is_sparse:
+                # creating csr directly would need plenty of manual work which
+                # would probably slow down the process - conversion coo to csr
+                # is fast
+                a = sp.coo_matrix(
+                    (np.hstack(data), (np.hstack(sp_row), np.hstack(sp_col))),
+                    shape=(n_rows, len(src_cols)),
+                    dtype=dtype
+                )
                 a = a.tocsr()
 
             return a
@@ -413,10 +446,7 @@ class Table(Sequence, Storage):
                 return table
 
             if isinstance(row_indices, slice):
-                start, stop, stride = row_indices.indices(source.X.shape[0])
-                n_rows = (stop - start) // stride
-                if n_rows < 0:
-                    n_rows = 0
+                n_rows = len(range(*row_indices.indices(source.X.shape[0])))
             elif row_indices is ...:
                 n_rows = len(source)
             else:
@@ -424,7 +454,7 @@ class Table(Sequence, Storage):
 
             self = cls()
             self.domain = domain
-            conversion = domain.get_conversion(source.domain)
+            conversion = DomainConversion(source.domain, domain)
             self.X = get_columns(row_indices, conversion.attributes, n_rows,
                                  is_sparse=conversion.sparse_X,
                                  variables=domain.attributes)
@@ -437,7 +467,7 @@ class Table(Sequence, Storage):
 
             dtype = np.float64
             if any(isinstance(var, StringVariable) for var in domain.metas):
-                dtype = np.object
+                dtype = object
             self.metas = get_columns(row_indices, conversion.metas,
                                      n_rows, dtype,
                                      is_sparse=conversion.sparse_metas,
@@ -1209,7 +1239,10 @@ class Table(Sequence, Storage):
         def get_col_indices():
             cols = chain(self.domain.variables, self.domain.metas)
             if isinstance(filter, IsDefined):
-                return list(cols)
+                if filter.columns is not None:
+                    return list(filter.columns)
+                else:
+                    return list(cols)
 
             if filter.column is not None:
                 return [filter.column]
@@ -1229,7 +1262,7 @@ class Table(Sequence, Storage):
                 if self.domain[col_idx].is_primitive():
                     return ~np.isnan(col.astype(float))
                 else:
-                    return col.astype(np.bool)
+                    return col.astype(bool)
             if isinstance(filter, FilterDiscrete):
                 return self._discrete_filter_to_indicator(filter, col)
             if isinstance(filter, FilterContinuous):
@@ -1249,11 +1282,14 @@ class Table(Sequence, Storage):
 
         col_indices = get_col_indices()
         if len(col_indices) == 1:
-            return col_filter(col_indices[0])
+            sel = col_filter(col_indices[0])
+        else:
+            sel = np.ones(len(self), dtype=bool)
+            for col_idx in col_indices:
+                sel *= col_filter(col_idx)
 
-        sel = np.ones(len(self), dtype=bool)
-        for col_idx in col_indices:
-            sel *= col_filter(col_idx)
+        if isinstance(filter, IsDefined) and filter.negate:
+            sel = ~sel
         return sel
 
     def _discrete_filter_to_indicator(self, filter, col):
@@ -1478,8 +1514,8 @@ class Table(Sequence, Storage):
 
         if any(not (var.is_discrete or var.is_continuous)
                for var in col_desc):
-            raise ValueError("contingency can be computed only for discrete "
-                             "and continuous values")
+            raise ValueError("Contingency can be computed only for categorical "
+                             "and numeric values.")
 
         # when we select a column in sparse matrix it is still two dimensional
         # and sparse - since it is just a column we can afford to transform
@@ -1568,7 +1604,7 @@ class Table(Sequence, Storage):
             names = get_unique_names_duplicates(names)
             attributes = [ContinuousVariable(name) for name in names]
         else:
-            places = int(np.ceil(np.log10(n_cols)))
+            places = int(np.ceil(np.log10(n_cols))) if n_cols else 1
             attributes = [ContinuousVariable(f"{feature_name} {i:0{places}}")
                           for i in range(1, n_cols + 1)]
         if old_domain is not None and feature_names_column:
@@ -1712,6 +1748,8 @@ def _check_arrays(*arrays, dtype=None, shape_1=None):
                              % (ninstances(array), shape_1))
 
         if sp.issparse(array):
+            if not (sp.isspmatrix_csr(array) or sp.isspmatrix_csc(array)):
+                array = array.tocsr()
             array.data = np.asarray(array.data)
             has_inf = _check_inf(array.data)
         else:
@@ -1762,7 +1800,7 @@ def _optimize_indices(indices, maxlen):
 
     if len(indices) >= 1:
         indices = np.asarray(indices)
-        if indices.dtype != np.bool:
+        if indices.dtype != bool:
             begin = indices[0]
             end = indices[-1]
             steps = np.diff(indices) if len(indices) > 1 else np.array([1])
@@ -1829,7 +1867,7 @@ def assure_domain_conversion_sparsity(target, source):
         Table: with fixed sparsity. The sparsity is set as it is recommended by domain conversion
             for transformation from source to the target domain.
     """
-    conversion = target.domain.get_conversion(source.domain)
+    conversion = DomainConversion(source.domain, target.domain)
     match_density = [assure_array_dense, assure_array_sparse]
     target.X = match_density[conversion.sparse_X](target.X)
     target.Y = match_density[conversion.sparse_Y](target.Y)
