@@ -15,6 +15,7 @@ from Orange.widgets import gui, widget
 from Orange.widgets.settings import DomainContextHandler, ContextSetting
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.widget import Msg, Input, Output
 
 
@@ -85,6 +86,17 @@ class ValueFromStringSubstring(Transformation):
         res[nans] = np.nan
         return res
 
+    def __eq__(self, other):
+        return super().__eq__(other) \
+               and self.patterns == other.patterns \
+               and self.case_sensitive == other.case_sensitive \
+               and self.match_beginning == other.match_beginning
+
+    def __hash__(self):
+        return hash((type(self), self.variable,
+                     tuple(self.patterns),
+                     self.case_sensitive, self.match_beginning))
+
 
 class ValueFromDiscreteSubstring(Lookup):
     """
@@ -128,7 +140,7 @@ class OWCreateClass(widget.OWWidget):
     description = "从字符串属性创类别属性"
     icon = "icons/CreateClass.svg"
     category = "Data"
-    keywords = []
+    keywords = ['chuangjian', 'leibie', 'fenlei']
 
     class Inputs:
         data = Input("数据(Data)", Table, replaces=['Data'])
@@ -180,7 +192,8 @@ class OWCreateClass(widget.OWWidget):
 
         combo = gui.comboBox(
             self.controlArea, self, "attribute", label="来自列: ",
-            box=True, orientation=Qt.Horizontal, callback=self.update_rules,
+            box=True, orientation=Qt.Horizontal, searchable=True,
+            callback=self.update_rules,
             model=DomainModel(valid_types=(StringVariable, DiscreteVariable)))
         # Don't use setSizePolicy keyword argument here: it applies to box,
         # not the combo
@@ -224,6 +237,9 @@ class OWCreateClass(widget.OWWidget):
         gui.button(box, self, "应用", autoDefault=False, width=180,
                    callback=self.apply)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         # TODO: Resizing upon changing the number of rules does not work
         self.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Maximum)
 
@@ -249,12 +265,16 @@ class OWCreateClass(widget.OWWidget):
         self.closeContext()
         self.rules = {}
         self.data = data
+        summary = len(data) if data else self.info.NoInput
+        details = format_summary_details(data) if data else ""
+        self.info.set_input_summary(summary, details)
         model = self.controls.attribute.model()
         model.set_domain(data and data.domain)
         self.Warning.no_nonnumeric_vars(shown=data is not None and not model)
         if not model:
             self.attribute = None
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return
         self.attribute = model[0]
         self.openContext(data)
@@ -461,6 +481,7 @@ class OWCreateClass(widget.OWWidget):
         self.class_name = self.class_name.strip()
         if not self.attribute:
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return
         domain = self.data.domain
         if not self.class_name:
@@ -469,11 +490,15 @@ class OWCreateClass(widget.OWWidget):
             self.Error.class_name_duplicated()
         if not self.class_name or self.class_name in domain:
             self.Outputs.data.send(None)
+            self.info.set_output_summary(self.info.NoOutput)
             return
         new_class = self._create_variable()
         new_domain = Domain(
             domain.attributes, new_class, domain.metas + domain.class_vars)
         new_data = self.data.transform(new_domain)
+        summary = len(new_data) if new_data is not None else self.info.NoOutput
+        details = format_summary_details(new_data) if new_data is not None else ""
+        self.info.set_output_summary(summary, details)
         self.Outputs.data.send(new_data)
 
     def _create_variable(self):
