@@ -4,10 +4,12 @@ import code
 import keyword
 import itertools
 import unicodedata
+import weakref
 from functools import reduce
-from collections import defaultdict
 from unittest.mock import patch
 
+from typing import Optional, List, TYPE_CHECKING
+
 from AnyQt.QtWidgets import (
     QPlainTextEdit, QListView, QSizePolicy, QMenu, QSplitter, QLineEdit,
     QAction, QToolButton, QFileDialog, QStyledItemDelegate,
@@ -17,7 +19,7 @@ from AnyQt.QtGui import (
     QColor, QBrush, QPalette, QFont, QTextDocument,
     QSyntaxHighlighter, QTextCharFormat, QTextCursor, QKeySequence,
 )
-from AnyQt.QtCore import Qt, QRegExp, QByteArray, QItemSelectionModel
+from AnyQt.QtCore import Qt, QRegExp, QByteArray, QItemSelectionModel, QSize
 
 from Orange.data import Table
 from Orange.base import Learner, Model
@@ -28,6 +30,9 @@ from Orange.widgets.settings import Setting
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.widget import OWWidget, Input, Output
 
+if TYPE_CHECKING:
+    from typing_extensions import TypedDict
+
 __all__ = ["OWPythonScript"]
 
 
@@ -339,10 +344,17 @@ class Script:
         self.flags = flags
         self.filename = filename
 
+    def asdict(self) -> '_ScriptData':
+        return dict(name=self.name, script=self.script, filename=self.filename)
+
+    @classmethod
+    def fromdict(cls, state: '_ScriptData') -> 'Script':
+        return Script(state["name"], state["script"], filename=state["filename"])
+
 
 class ScriptItemDelegate(QStyledItemDelegate):
-    @staticmethod
-    def displayText(script, _locale):
+    # pylint: disable=no-self-use
+    def displayText(self, script, _locale):
         if script.flags & Script.Modified:
             return "*" + script.name
         else:
@@ -357,17 +369,14 @@ class ScriptItemDelegate(QStyledItemDelegate):
             option.palette.setColor(QPalette.Highlight, QColor(Qt.darkRed))
         super().paint(painter, option, index)
 
-    @staticmethod
-    def createEditor(parent, _option, _index):
+    def createEditor(self, parent, _option, _index):
         return QLineEdit(parent)
 
-    @staticmethod
-    def setEditorData(editor, index):
+    def setEditorData(self, editor, index):
         script = index.data(Qt.DisplayRole)
         editor.setText(script.name)
 
-    @staticmethod
-    def setModelData(editor, model, index):
+    def setModelData(self, editor, model, index):
         model[index.row()].name = str(editor.text())
 
 
@@ -380,38 +389,48 @@ def select_row(view, row):
                     QItemSelectionModel.ClearAndSelect)
 
 
+if TYPE_CHECKING:
+    # pylint: disable=used-before-assignment
+    _ScriptData = TypedDict("_ScriptData", {
+        "name": str, "script": str, "filename": Optional[str]
+    })
+
+
 class OWPythonScript(OWWidget):
     name = "python脚本(Python Script)"
     description = "编写一个 python 脚本并在输入数据或模型上运行它。"
     icon = "icons/PythonScript.svg"
     priority = 3150
-    keywords = ["file", "program"]
+    keywords = ["file", "program", 'jiaoben']
+    category = "Data"
 
     class Inputs:
-        data = Input("数据(Data)", Table, replaces=["in_data", 'Data'],
+        data = Input("数据(Data)", Table, replaces=['Data', "in_data"],
                      default=True, multiple=True)
-        learner = Input("学习器(Learner)", Learner, replaces=["in_learner", 'Learner'],
+        learner = Input("学习器(Learner)", Learner, replaces=['Learner', "in_learner"],
                         default=True, multiple=True)
-        classifier = Input("分类器(Classifier)", Model, replaces=["in_classifier", 'Classifier'],
+        classifier = Input("分类器(Classifier)", Model, replaces=['Classifier', "in_classifier"],
                            default=True, multiple=True)
-        object = Input("对象(Object)", object, replaces=["in_object", 'Object'],
+        object = Input("对象(Object)", object, replaces=['Object', "in_object"],
                        default=False, multiple=True)
 
     class Outputs:
-        data = Output("数据(Data)", Table, replaces=["out_data", 'Data'])
-        learner = Output("学习器(Learner)", Learner, replaces=["out_learner", 'Learner'])
-        classifier = Output("分类器(Classifier)", Model, replaces=["out_classifier", 'Classifier'])
-        object = Output("对象(Object)", object, replaces=["out_object", 'Object'])
+        data = Output("数据(Data)", Table, replaces=['Data', "out_data"])
+        learner = Output("学习器(Learner)", Learner, replaces=['Learner', "out_learner"])
+        classifier = Output("分类器(Classifier)", Model, replaces=['Classifier', "out_classifier"])
+        object = Output("对象(Object)", object, replaces=['Object', "out_object"])
 
     signal_names = ("data", "learner", "classifier", "object")
-    signal_names2 = ("数据", "学习器", "分类器", "对象")
-    libraryListSource: list
 
-    libraryListSource = \
-        Setting([Script("Hello world", "print('Hello world')\n")])
+    settings_version = 2
+    scriptLibrary: 'List[_ScriptData]' = Setting([{
+        "name": "Hello world",
+        "script": "print('Hello world')\n",
+        "filename": None
+    }])
     currentScriptIndex = Setting(0)
-    scriptText = Setting(None, schema_only=True)
-    splitterState = Setting(None)
+    scriptText: Optional[str] = Setting(None, schema_only=True)
+    splitterState: Optional[bytes] = Setting(None)
 
     # Widgets in the same schema share namespace through a dictionary whose
     # key is self.signalManager. ales-erjavec expressed concern (and I fully
@@ -419,29 +438,27 @@ class OWPythonScript(OWWidget):
     # anyway. If this causes any problems in the future, replace this with
     # shared_namespaces = {} and thus use a common namespace for all instances
     # of # PythonScript even if they are in different schemata.
-    shared_namespaces = defaultdict(dict)
+    shared_namespaces = weakref.WeakKeyDictionary()
 
     class Error(OWWidget.Error):
         pass
 
     def __init__(self):
         super().__init__()
+        self.libraryListSource = []
 
         for name in self.signal_names:
             setattr(self, name, {})
 
-        for s in self.libraryListSource:
-            s.flags = 0
-
         self._cachedDocuments = {}
 
         self.infoBox = gui.vBox(self.controlArea, '信息')
         gui.label(
             self.infoBox, self,
             "<p>执行python脚本.</p><p>输入变量:<ul><li> " +
-            "<li>".join(map("输入{0}, 输入{0}".format, self.signal_names2)) +
+            "<li>".join(map("in_{0}, in_{0}s".format, self.signal_names)) +
             "</ul></p><p>输出变量:<ul><li>" +
-            "<li>".join(map("输出{0}".format, self.signal_names2)) +
+            "<li>".join(map("out_{0}".format, self.signal_names)) +
             "</ul></p>"
         )
 
@@ -544,34 +561,36 @@ class OWPythonScript(OWWidget):
         self.console.document().setDefaultFont(QFont(defaultFont))
         self.consoleBox.setAlignment(Qt.AlignBottom)
         self.console.setTabStopWidth(4)
+        self.splitCanvas.setSizes([2, 1])
+        self.setAcceptDrops(True)
+        self.controlArea.layout().addStretch(10)
 
+        self._restoreState()
+        self.settingsAboutToBePacked.connect(self._saveState)
+
+    def sizeHint(self) -> QSize:
+        return super().sizeHint().expandedTo(QSize(800, 600))
+
+    def _restoreState(self):
+        self.libraryListSource = [Script.fromdict(s) for s in self.scriptLibrary]
+        self.libraryList.wrap(self.libraryListSource)
         select_row(self.libraryView, self.currentScriptIndex)
 
-        self.restoreScriptText()
-        self.settingsAboutToBePacked.connect(self.saveScriptText)
-
-        self.splitCanvas.setSizes([2, 1])
-        if self.splitterState is not None:
-            self.splitCanvas.restoreState(QByteArray(self.splitterState))
-
-        self.setAcceptDrops(True)
-
-        self.splitCanvas.splitterMoved[int, int].connect(self.onSpliterMoved)
-        self.controlArea.layout().addStretch(1)
-        self.resize(800, 600)
-
-    def restoreScriptText(self):
         if self.scriptText is not None:
             current = self.text.toPlainText()
             # do not mark scripts as modified
             if self.scriptText != current:
                 self.text.document().setPlainText(self.scriptText)
 
-    def saveScriptText(self):
+        if self.splitterState is not None:
+            self.splitCanvas.restoreState(QByteArray(self.splitterState))
+
+    def _saveState(self):
+        self.scriptLibrary = [s.asdict() for s in self.libraryListSource]
         self.scriptText = self.text.toPlainText()
+        self.splitterState = bytes(self.splitCanvas.saveState())
 
     def handle_input(self, obj, sig_id, signal):
-        sig_id = sig_id[0]
         dic = getattr(self, signal)
         if obj is None:
             if sig_id in dic.keys():
@@ -675,9 +694,6 @@ class OWPythonScript(OWWidget):
             self.libraryList[index].flags = Script.Modified if modified else 0
             self.libraryList.emitDataChanged(index)
 
-    def onSpliterMoved(self, _pos, _ind):
-        self.splitterState = bytes(self.splitCanvas.saveState())
-
     def restoreSaved(self):
         index = self.selectedScriptIndex()
         if index is not None:
@@ -711,7 +727,7 @@ class OWPythonScript(OWWidget):
             f.close()
 
     def initial_locals_state(self):
-        d = self.shared_namespaces[self.signalManager].copy()
+        d = self.shared_namespaces.setdefault(self.signalManager, {}).copy()
         for name in self.signal_names:
             value = getattr(self, name)
             all_values = list(value.values())
@@ -724,7 +740,7 @@ class OWPythonScript(OWWidget):
         not_saved = reduce(set.union,
                            ({f"in_{name}s", f"in_{name}", f"out_{name}"}
                             for name in self.signal_names))
-        self.shared_namespaces[self.signalManager].update(
+        self.shared_namespaces.setdefault(self.signalManager, {}).update(
             {name: value for name, value in namespace.items()
              if name not in not_saved})
 
@@ -748,8 +764,7 @@ class OWPythonScript(OWWidget):
                 out_var = None
             getattr(self.Outputs, signal).send(out_var)
 
-    @staticmethod
-    def dragEnterEvent(event):
+    def dragEnterEvent(self, event):  # pylint: disable=no-self-use
         urls = event.mimeData().urls()
         if urls:
             # try reading the file as text
@@ -763,6 +778,14 @@ class OWPythonScript(OWWidget):
         if urls:
             self.text.pasteFile(urls[0])
 
+    @classmethod
+    def migrate_settings(cls, settings, version):
+        if version is not None and version < 2:
+            scripts = settings.pop("libraryListSource")  # type: List[Script]
+            library = [dict(name=s.name, script=s.script, filename=s.filename)
+                       for s in scripts]  # type: List[_ScriptData]
+            settings["scriptLibrary"] = library
+
 
 if __name__ == "__main__":  # pragma: no cover
     WidgetPreview(OWPythonScript).run()
