@@ -1,43 +1,39 @@
-"""
-Rank
-====
-
-Rank (score) features for prediction.
-
-"""
-import warnings
-from collections import namedtuple, OrderedDict
 import logging
+import warnings
+from collections import OrderedDict, namedtuple
 from functools import partial
 from itertools import chain
+from types import SimpleNamespace
+from typing import Any, Callable, List, Tuple
 
 import numpy as np
-from scipy.sparse import issparse
-
+from AnyQt.QtCore import (
+    QItemSelection, QItemSelectionModel, QItemSelectionRange, Qt
+)
 from AnyQt.QtGui import QFontMetrics
 from AnyQt.QtWidgets import (
-    QTableView, QRadioButton, QButtonGroup, QGridLayout,
-    QStackedWidget, QHeaderView, QCheckBox, QItemDelegate,
-)
-from AnyQt.QtCore import (
-    Qt, QItemSelection, QItemSelectionRange, QItemSelectionModel,
+    QButtonGroup, QCheckBox, QGridLayout, QHeaderView, QItemDelegate,
+    QRadioButton, QStackedWidget, QTableView
 )
+from orangewidget.settings import IncompatibleContext
+from scipy.sparse import issparse
 
-from Orange.data import (Table, Domain, ContinuousVariable, DiscreteVariable,
-                         StringVariable)
-from Orange.misc.cache import memoize_method
+from Orange.data import (
+    ContinuousVariable, DiscreteVariable, Domain, StringVariable, Table
+)
+from Orange.data.util import get_unique_names_duplicates
 from Orange.preprocess import score
-from Orange.widgets import report
-from Orange.widgets import gui
-from Orange.widgets.settings import (DomainContextHandler, Setting,
-                                     ContextSetting)
+from Orange.widgets import gui, report
+from Orange.widgets.settings import (
+    ContextSetting, DomainContextHandler, Setting
+)
+from Orange.widgets.unsupervised.owdistances import InterruptException
+from Orange.widgets.utils.concurrent import ConcurrentWidgetMixin, TaskState
 from Orange.widgets.utils.itemmodels import PyTableModel
 from Orange.widgets.utils.sql import check_sql_input
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.utils.widgetpreview import WidgetPreview
-from Orange.widgets.widget import (
-    OWWidget, Msg, Input, Output, AttributeList
-)
-
+from Orange.widgets.widget import AttributeList, Input, Msg, Output, OWWidget
 
 log = logging.getLogger(__name__)
 
@@ -51,34 +47,33 @@ class ProblemType:
                 cls.REGRESSION if isinstance(variable, ContinuousVariable) else
                 cls.UNSUPERVISED)
 
-ScoreMeta = namedtuple("score_meta", ["name", "shortname", "scorer", 'problem_type', 'is_default'])
+ScoreMeta = namedtuple("score_meta", ["name", "zh_name", "shortname", "zh_shortname", "scorer", 'problem_type', 'is_default'])
 
 # Default scores.
 CLS_SCORES = [
-    ScoreMeta("信息增益(Information Gain)", "信息增益",
+    ScoreMeta("Information Gain", "信息增益", "Info. gain", "信息增益",
               score.InfoGain, ProblemType.CLASSIFICATION, False),
-    ScoreMeta("信息增益比(Information Gain Ratio)", "增益比",
+    ScoreMeta("Information Gain Ratio", "信息增益比", "Gain ratio", "增益比",
               score.GainRatio, ProblemType.CLASSIFICATION, True),
-    ScoreMeta("基尼下降(Gini Decrease)", "基尼",
+    ScoreMeta("Gini Decrease", "基尼下降", "Gini", "基尼",
               score.Gini, ProblemType.CLASSIFICATION, True),
-    ScoreMeta("ANOVA", "ANOVA",
+    ScoreMeta("ANOVA", "ANOVA", "ANOVA", "ANOVA",
               score.ANOVA, ProblemType.CLASSIFICATION, False),
-    ScoreMeta("χ²", "χ²",
+    ScoreMeta("χ²", "χ²", "χ²", "χ²",
               score.Chi2, ProblemType.CLASSIFICATION, False),
-    ScoreMeta("ReliefF", "ReliefF",
+    ScoreMeta("ReliefF", "ReliefF", "ReliefF", "ReliefF",
               score.ReliefF, ProblemType.CLASSIFICATION, False),
-    ScoreMeta("FCBF", "FCBF",
+    ScoreMeta("FCBF", "FCBF", "FCBF", "FCBF",
               score.FCBF, ProblemType.CLASSIFICATION, False)
 ]
 REG_SCORES = [
-    ScoreMeta("单变量回归(Univariate Regression)", "单变量回归",
+    ScoreMeta("Univariate Regression", "单变量回归", "Univar. reg.", "单变量回归",
               score.UnivariateLinearRegression, ProblemType.REGRESSION, True),
-    ScoreMeta("RReliefF", "RReliefF",
+    ScoreMeta("RReliefF", "RReliefF", "RReliefF", "RReliefF",
               score.RReliefF, ProblemType.REGRESSION, True)
 ]
 SCORES = CLS_SCORES + REG_SCORES
 
-
 class TableView(QTableView):
     def __init__(self, parent=None, **kwargs):
         super().__init__(parent=parent,
@@ -171,12 +166,85 @@ class TableModel(PyTableModel):
         return indices
 
 
-class OWRank(OWWidget):
+class Results(SimpleNamespace):
+    method_scores: Tuple[ScoreMeta, np.ndarray] = None
+    scorer_scores: Tuple[ScoreMeta, Tuple[np.ndarray, List[str]]] = None
+
+
+def get_method_scores(data: Table, method: ScoreMeta) -> np.ndarray:
+    estimator = method.scorer()
+    # The widget handles infs and nans.
+    # Any errors in scorers need to be detected elsewhere.
+    with np.errstate(all="ignore"):
+        try:
+            scores = np.asarray(estimator(data))
+        except ValueError:
+            try:
+                scores = np.array(
+                    [estimator(data, attr) for attr in data.domain.attributes]
+                )
+            except ValueError:
+                log.error("%s doesn't work on this data", method.name)
+                scores = np.full(len(data.domain.attributes), np.nan)
+            else:
+                log.warning(
+                    "%s had to be computed separately for each " "variable",
+                    method.name,
+                )
+        return scores
+
+
+def get_scorer_scores(
+    data: Table, scorer: ScoreMeta
+) -> Tuple[np.ndarray, Tuple[str]]:
+    try:
+        scores = scorer.scorer.score_data(data).T
+    except (ValueError, TypeError):
+        log.error("%s doesn't work on this data", scorer.name)
+        scores = np.full((len(data.domain.attributes), 1), np.nan)
+
+    labels = (
+        (scorer.shortname,)
+        if scores.shape[1] == 1
+        else tuple(
+            scorer.shortname + "_" + str(i)
+            for i in range(1, 1 + scores.shape[1])
+        )
+    )
+    return scores, labels
+
+
+def run(
+    data: Table,
+    methods: List[ScoreMeta],
+    scorers: List[ScoreMeta],
+    state: TaskState,
+) -> Results:
+    progress_steps = iter(np.linspace(0, 100, len(methods) + len(scorers)))
+
+    def call_with_cb(get_scores: Callable, method: ScoreMeta):
+        scores = get_scores(data, method)
+        state.set_progress_value(next(progress_steps))
+        if state.is_interruption_requested():
+            raise InterruptException
+        return scores
+
+    method_scores = tuple(
+        (method, call_with_cb(get_method_scores, method)) for method in methods
+    )
+    scorer_scores = tuple(
+        (scorer, call_with_cb(get_scorer_scores, scorer)) for scorer in scorers
+    )
+    return Results(method_scores=method_scores, scorer_scores=scorer_scores)
+
+
+class OWRank(OWWidget, ConcurrentWidgetMixin):
     name = "排名(Rank)"
     description = "根据数据特征的相关性对其进行排名和筛选。"
     icon = "icons/Rank.svg"
     priority = 1102
-    keywords = []
+    keywords = ['paiming', 'mingci', 'paixu']
+    category = "Data"
 
     buttons_area_orientation = Qt.Vertical
 
@@ -186,7 +254,7 @@ class OWRank(OWWidget):
 
     class Outputs:
         reduced_data = Output("选中的数据(Reduced Data)", Table, default=True, replaces=['Reduced Data'])
-        scores = Output("分数(Scores)", Table, replaces=['Scores'])
+        scores = Output("评分(Scores)", Table, replaces=['Scores'])
         features = Output("特征(Features)", AttributeList, dynamic=False, replaces=['Features'])
 
     SelectNone, SelectAll, SelectManual, SelectNBest = range(4)
@@ -197,15 +265,13 @@ class OWRank(OWWidget):
     sorting = Setting((0, Qt.DescendingOrder))
     selected_methods = Setting(set())
 
-    settings_version = 2
+    settings_version = 3
     settingsHandler = DomainContextHandler()
-    selected_rows = ContextSetting([])
+    selected_attrs = ContextSetting([], schema_only=True)
     selectionMethod = ContextSetting(SelectNBest)
 
     class Information(OWWidget.Information):
-        no_target_var = Msg("Data does not have a single target variable. "
-                            "You can still connect in unsupervised scorers "
-                            "such as PCA.")
+        no_target_var = Msg("Data does not have a (single) target variable.")
         missings_imputed = Msg('Missing values will be imputed as needed.')
 
     class Error(OWWidget.Error):
@@ -213,19 +279,27 @@ class OWRank(OWWidget):
         inadequate_learner = Msg("Scorer {} inadequate: {}")
         no_attributes = Msg("Data does not have a single attribute.")
 
+    class Warning(OWWidget.Warning):
+        renamed_variables = Msg(
+            "Variables with duplicated names have been renamed.")
+
     def __init__(self):
-        super().__init__()
+        OWWidget.__init__(self)
+        ConcurrentWidgetMixin.__init__(self)
         self.scorers = OrderedDict()
         self.out_domain_desc = None
         self.data = None
         self.problem_type_mode = ProblemType.CLASSIFICATION
 
+        # results caches
+        self.scorers_results = {}
+        self.methods_results = {}
+
         if not self.selected_methods:
             self.selected_methods = {method.name for method in SCORES
                                      if method.is_default}
 
         # GUI
-
         self.ranksModel = model = TableModel(parent=self)  # type: TableModel
         self.ranksView = view = TableView(self)            # type: TableView
         self.mainArea.layout().addWidget(view)
@@ -250,7 +324,7 @@ class OWRank(OWWidget):
             stacked.addWidget(box)
             for method in scoring_methods:
                 box.layout().addWidget(QCheckBox(
-                    method.name, self,
+                    method.zh_name, self,
                     objectName=method.shortname,  # To be easily found in tests
                     checked=method.name in self.selected_methods,
                     stateChanged=partial(self.methodSelectionChanged, method_name=method.name)))
@@ -293,6 +367,9 @@ class OWRank(OWWidget):
 
         gui.auto_send(selMethBox, self, "auto_apply", box=False)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         self.resize(690, 500)
 
     def switchProblemType(self, index):
@@ -306,12 +383,13 @@ class OWRank(OWWidget):
     @check_sql_input
     def set_data(self, data):
         self.closeContext()
-        self.selected_rows = []
+        self.selected_attrs = []
         self.ranksModel.clear()
         self.ranksModel.resetSorting(True)
 
-        self.get_method_scores.cache_clear()  # pylint: disable=no-member
-        self.get_scorer_scores.cache_clear()  # pylint: disable=no-member
+        self.scorers_results = {}
+        self.methods_results = {}
+        self.cancel()
 
         self.Error.clear()
         self.Information.clear()
@@ -325,6 +403,8 @@ class OWRank(OWWidget):
         self.switchProblemType(ProblemType.CLASSIFICATION)
         if self.data is not None:
             domain = self.data.domain
+            self.info.set_input_summary(len(self.data),
+                                        format_summary_details(self.data))
 
             if domain.has_discrete_class:
                 problem_type = ProblemType.CLASSIFICATION
@@ -346,13 +426,15 @@ class OWRank(OWWidget):
                 max((a.name for a in domain.attributes), key=len))
 
             self.selectionMethod = OWRank.SelectNBest
+        else:
+            self.info.set_input_summary(self.info.NoInput)
 
         self.openContext(data)
         self.selectButtons.button(self.selectionMethod).setChecked(True)
 
     def handleNewSignals(self):
         self.setStatusMessage('Running')
-        self.updateScores()
+        self.update_scores()
         self.setStatusMessage('')
         self.on_select()
 
@@ -364,83 +446,75 @@ class OWRank(OWWidget):
             # Avoid caching a (possibly stale) previous instance of the same
             # Scorer passed via the same signal
             if id in self.scorers:
-                # pylint: disable=no-member
-                self.get_scorer_scores.cache_clear()
+                self.scorers_results = {}
 
-            self.scorers[id] = ScoreMeta(scorer.name, scorer.name, scorer,
+            self.scorers[id] = ScoreMeta(scorer.name, scorer.name, scorer.name, scorer.name, scorer,
                                          ProblemType.from_variable(scorer.class_type),
                                          False)
 
-    @memoize_method()
-    def get_method_scores(self, method):
-        # These errors often happen, but they result in nans, which
-        # are handled correctly by the widget
-        estimator = method.scorer()
-        data = self.data
-        try:
-            scores = np.asarray(estimator(data))
-        except ValueError:
-            try:
-                scores = np.array([estimator(data, attr)
-                                   for attr in data.domain.attributes])
-            except ValueError:
-                log.error("%s doesn't work on this data", method.name)
-                scores = np.full(len(data.domain.attributes), np.nan)
+    def _get_methods(self):
+        return [
+            method
+            for method in SCORES
+            if (
+                method.name in self.selected_methods
+                and method.problem_type == self.problem_type_mode
+                and (
+                    not issparse(self.data.X)
+                    or method.scorer.supports_sparse_data
+                )
+            )
+        ]
+
+    def _get_scorers(self):
+        scorers = []
+        for scorer in self.scorers.values():
+            if scorer.problem_type in (
+                self.problem_type_mode,
+                ProblemType.UNSUPERVISED,
+            ):
+                scorers.append(scorer)
             else:
-                log.warning("%s had to be computed separately for each "
-                            "variable", method.name)
-        return scores
+                self.Error.inadequate_learner(
+                    scorer.name, scorer.learner_adequacy_err_msg
+                )
+        return scorers
 
-    @memoize_method()
-    def get_scorer_scores(self, scorer):
-        try:
-            scores = scorer.scorer.score_data(self.data).T
-        except ValueError:
-            log.error("%s doesn't work on this data", scorer.name)
-            scores = np.full((len(self.data.domain.attributes), 1), np.nan)
-
-        labels = ((scorer.shortname,)
-                  if scores.shape[1] == 1 else
-                  tuple(scorer.shortname + '_' + str(i)
-                        for i in range(1, 1 + scores.shape[1])))
-        return scores, labels
-
-    def updateScores(self):
+    def update_scores(self):
         if self.data is None:
             self.ranksModel.clear()
             self.Outputs.scores.send(None)
             return
 
-        methods = [method
-                   for method in SCORES
-                   if (method.name in self.selected_methods and
-                       method.problem_type == self.problem_type_mode and
-                       (not issparse(self.data.X) or
-                        method.scorer.supports_sparse_data))]
-
-        scorers = []
         self.Error.inadequate_learner.clear()
-        for scorer in self.scorers.values():
-            if scorer.problem_type in (self.problem_type_mode, ProblemType.UNSUPERVISED):
-                scorers.append(scorer)
-            else:
-                self.Error.inadequate_learner(scorer.name, scorer.learner_adequacy_err_msg)
 
-        method_scores = tuple(self.get_method_scores(method)
-                              for method in methods)
+        scorers = [
+            s for s in self._get_scorers() if s not in self.scorers_results
+        ]
+        methods = [
+            m for m in self._get_methods() if m not in self.methods_results
+        ]
+        self.start(run, self.data, methods, scorers)
 
-        scorer_scores, scorer_labels = (), ()
-        if scorers:
-            scorer_scores, scorer_labels = zip(*(self.get_scorer_scores(scorer)
-                                                 for scorer in scorers))
-            scorer_labels = tuple(chain.from_iterable(scorer_labels))
+    def on_done(self, result: Results) -> None:
+        self.methods_results.update(result.method_scores)
+        self.scorers_results.update(result.scorer_scores)
 
-        labels = tuple(method.shortname for method in methods) + scorer_labels
+        methods = self._get_methods()
+        method_labels = tuple(m.shortname for m in methods)
+        method_scores = tuple(self.methods_results[m] for m in methods)
+
+        scores = [self.scorers_results[s] for s in self._get_scorers()]
+        scorer_scores, scorer_labels = zip(*scores) if scores else ((), ())
+
+        labels = method_labels + tuple(chain.from_iterable(scorer_labels))
         model_array = np.column_stack(
-            ([len(a.values) if a.is_discrete else np.nan
-              for a in self.data.domain.attributes],) +
-            (method_scores if method_scores else ()) +
-            (scorer_scores if scorer_scores else ())
+            (
+                [len(a.values) if a.is_discrete else np.nan
+                 for a in self.data.domain.attributes],
+            )
+            + method_scores
+            + scorer_scores
         )
         for column, values in enumerate(model_array.T):
             self.ranksModel.setExtremesFrom(column, values)
@@ -455,17 +529,27 @@ class OWRank(OWWidget):
             if sort_column < len(labels):
                 # adds 1 for '#' (discrete count) column
                 self.ranksModel.sort(sort_column + 1, sort_order)
-                self.ranksView.horizontalHeader().setSortIndicator(sort_column + 1, sort_order)
+                self.ranksView.horizontalHeader().setSortIndicator(
+                    sort_column + 1, sort_order
+                )
         except ValueError:
             pass
 
         self.autoSelection()
         self.Outputs.scores.send(self.create_scores_table(labels))
 
+    def on_exception(self, ex: Exception) -> None:
+        raise ex
+
+    def on_partial_result(self, result: Any) -> None:
+        pass
+
     def on_select(self):
         # Save indices of attributes in the original, unsorted domain
-        self.selected_rows = list(self.ranksModel.mapToSourceRows([
-            i.row() for i in self.ranksView.selectionModel().selectedRows(0)]))
+        selected_rows = self.ranksView.selectionModel().selectedRows(0)
+        row_indices = [i.row() for i in selected_rows]
+        attr_indices = self.ranksModel.mapToSourceRows(row_indices)
+        self.selected_attrs = [self.data.domain[idx] for idx in attr_indices]
         self.commit()
 
     def setSelectionMethod(self, method):
@@ -494,8 +578,10 @@ class OWRank(OWWidget):
             )
         else:
             selection = QItemSelection()
-            if self.selected_rows is not None:
-                for row in model.mapFromSourceRows(self.selected_rows):
+            if self.selected_attrs is not None:
+                attr_indices = [self.data.domain.attributes.index(var)
+                                for var in self.selected_attrs]
+                for row in model.mapFromSourceRows(attr_indices):
                     selection.append(QItemSelectionRange(
                         model.index(row, 0), model.index(row, columnCount - 1)))
 
@@ -517,7 +603,7 @@ class OWRank(OWWidget):
         elif method_name in self.selected_methods:
             self.selected_methods.remove(method_name)
 
-        self.updateScores()
+        self.update_scores()
 
     def send_report(self):
         if not self.data:
@@ -528,29 +614,34 @@ class OWRank(OWWidget):
             self.report_items("Output", self.out_domain_desc)
 
     def commit(self):
-        selected_attrs = []
-        if self.data is not None:
-            selected_attrs = [self.data.domain.attributes[i]
-                              for i in self.selected_rows]
-        if not selected_attrs:
+        if not self.selected_attrs:
             self.Outputs.reduced_data.send(None)
             self.Outputs.features.send(None)
             self.out_domain_desc = None
+            self.info.set_output_summary(self.info.NoOutput)
         else:
             reduced_domain = Domain(
-                selected_attrs, self.data.domain.class_var, self.data.domain.metas)
+                self.selected_attrs, self.data.domain.class_var,
+                self.data.domain.metas)
             data = self.data.transform(reduced_domain)
             self.Outputs.reduced_data.send(data)
-            self.Outputs.features.send(AttributeList(selected_attrs))
+            self.Outputs.features.send(AttributeList(self.selected_attrs))
             self.out_domain_desc = report.describe_domain(data.domain)
+            self.info.set_output_summary(len(data),
+                                         format_summary_details(data))
 
     def create_scores_table(self, labels):
+        self.Warning.renamed_variables.clear()
         model_list = self.ranksModel.tolist()
         if not model_list or len(model_list[0]) == 1:  # Empty or just n_values column
             return None
+        unique, renamed = get_unique_names_duplicates(labels + ('Feature',),
+                                                      return_duplicated=True)
+        if renamed:
+            self.Warning.renamed_variables(', '.join(renamed))
 
-        domain = Domain([ContinuousVariable(label) for label in labels],
-                        metas=[StringVariable("Feature")])
+        domain = Domain([ContinuousVariable(label) for label in unique[:-1]],
+                        metas=[StringVariable(unique[-1])])
 
         # Prevent np.inf scores
         finfo = np.finfo(np.float64)
@@ -584,11 +675,10 @@ class OWRank(OWWidget):
 
     @classmethod
     def migrate_context(cls, context, version):
-        if version is None or version < 2:
-            # Old selection was saved as sorted indices. New selection is original indices.
-            # Since we can't devise the latter without first computing the ranks,
-            # just reset the selection to avoid confusion.
-            context.values['selected_rows'] = []
+        if version is None or version < 3:
+            # Selections were stored as indices, so these contexts matched
+            # any domain. The only safe thing to do is to remove them.
+            raise IncompatibleContext
 
 
 if __name__ == "__main__":  # pragma: no cover
@@ -604,4 +694,4 @@ if __name__ == "__main__":  # pragma: no cover
     WidgetPreview(OWRank).run(
         set_learner=(RandomForestLearner(), (3, 'Learner', None)),
         set_data=Table("heart_disease.tab"))
-"""
+    """
