@@ -1,8 +1,3 @@
-"""
-ROC Analysis Widget
--------------------
-
-"""
 import operator
 from functools import reduce, wraps
 from collections import namedtuple, deque, OrderedDict
@@ -10,22 +5,26 @@ from collections import namedtuple, deque, OrderedDict
 import numpy as np
 import sklearn.metrics as skl_metrics
 
-from AnyQt.QtWidgets import QListView, QLabel, QGridLayout, QFrame, QAction, QToolTip
-from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QPalette, QFont, QCursor
-from AnyQt.QtCore import Qt
+from AnyQt.QtWidgets import QListView, QLabel, QGridLayout, QFrame, QAction, \
+    QToolTip
+from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QPalette, QFont, \
+    QCursor, QFontMetrics
+from AnyQt.QtCore import Qt, QSize
 import pyqtgraph as pg
 
 import Orange
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.evaluate.contexthandlers import \
     EvaluationResultsContextHandler
-from Orange.widgets.evaluate.utils import \
-    check_results_adequacy, results_for_preview
+from Orange.widgets.evaluate.utils import check_results_adequacy
 from Orange.widgets.utils import colorpalettes
 from Orange.widgets.utils.widgetpreview import WidgetPreview
 from Orange.widgets.widget import Input
 from Orange.widgets import report
 
+from Orange.widgets.evaluate.utils import results_for_preview
+from Orange.evaluation.testing import Results
+
 
 #: Points on a ROC curve
 ROCPoints = namedtuple(
@@ -91,11 +90,11 @@ def roc_data_from_results(results, clf_index, target):
     :rval ROCData:
         A instance holding the computed curves.
     """
-    merged = roc_curve_for_fold(results, slice(0, -1), clf_index, target)
+    merged = roc_curve_for_fold(results, ..., clf_index, target)
     merged_curve = ROCCurve(ROCPoints(*merged),
                             ROCPoints(*roc_curve_convex_hull(merged)))
 
-    folds = results.folds if results.folds is not None else [slice(0, -1)]
+    folds = results.folds if results.folds is not None else [...]
     fold_curves = []
     for fold in folds:
         points = roc_curve_for_fold(results, fold, clf_index, target)
@@ -299,10 +298,11 @@ class OWROCAnalysis(widget.OWWidget):
     description = "根据分类器的评估结果显示接受者操作曲线。"
     icon = "icons/ROCAnalysis.svg"
     priority = 1010
-    keywords = []
+    keywords = ['fenxi']
+    category = 'evaluate'
 
     class Inputs:
-        evaluation_results = Input("评价结果", Orange.evaluation.Results)
+        evaluation_results = Input("评估结果(Evaluation Results)", Orange.evaluation.Results, replaces=["Evaluation Results"])
 
     settingsHandler = EvaluationResultsContextHandler()
     target_index = settings.ContextSetting(0)
@@ -338,32 +338,29 @@ class OWROCAnalysis(widget.OWWidget):
         self._tooltip_cache = None
 
         box = gui.vBox(self.controlArea, "绘制")
-        tbox = gui.vBox(box, "目标类别")
-        tbox.setFlat(True)
-
         self.target_cb = gui.comboBox(
-            tbox, self, "target_index", callback=self._on_target_changed,
-            contentsLength=8)
+            box, self, "target_index",
+            label="目标", orientation=Qt.Horizontal,
+            callback=self._on_target_changed,
+            contentsLength=8, searchable=True)
 
-        cbox = gui.vBox(box, "分类器")
-        cbox.setFlat(True)
+        gui.widgetLabel(box, "分类器")
+        line_height = 4 * QFontMetrics(self.font()).lineSpacing()
         self.classifiers_list_box = gui.listBox(
-            cbox, self, "selected_classifiers", "classifier_names",
+            box, self, "selected_classifiers", "classifier_names",
             selectionMode=QListView.MultiSelection,
-            callback=self._on_classifiers_changed)
+            callback=self._on_classifiers_changed,
+            sizeHint=QSize(0, line_height))
 
-        abox = gui.vBox(box, "从折叠(Folds)合并ROC曲线")
-        abox.setFlat(True)
+        abox = gui.vBox(self.controlArea, "曲线")
         gui.comboBox(abox, self, "roc_averaging",
-                     items=["从折叠合并预测", "平均真阳性率",
+                     items=["合并折叠预测", "平均真阳性率",
                             "阈值处的平均真阳性率和假阳性率", "显示单个曲线"],
                      callback=self._replot)
 
-        hbox = gui.vBox(box, "ROC凸包")
-        hbox.setFlat(True)
-        gui.checkBox(hbox, self, "display_convex_curve",
+        gui.checkBox(abox, self, "display_convex_curve",
                      "显示凸ROC曲线", callback=self._replot)
-        gui.checkBox(hbox, self, "display_convex_hull",
+        gui.checkBox(abox, self, "display_convex_hull",
                      "显示ROC凸包", callback=self._replot)
 
         box = gui.vBox(self.controlArea, "分析(Analysis)")
@@ -372,7 +369,7 @@ class OWROCAnalysis(widget.OWWidget):
                      "默认阈值（0.5）点",
                      callback=self._on_display_def_threshold_changed)
 
-        gui.checkBox(box, self, "display_perf_line", "显示性能行",
+        gui.checkBox(box, self, "display_perf_line", "显示性能线",
                      callback=self._on_display_perf_line_changed)
         grid = QGridLayout()
         gui.indentedBox(box, orientation=grid)
@@ -393,7 +390,7 @@ class OWROCAnalysis(widget.OWWidget):
                                         callback=self._on_target_prior_changed)
         self.target_prior_sp.setSuffix(" %")
         self.target_prior_sp.addAction(QAction("Auto", sp))
-        grid.addWidget(QLabel("目标类先验概率:"))
+        grid.addWidget(QLabel("先验概率:"))
         grid.addWidget(self.target_prior_sp, 2, 1)
 
         self.plotview = pg.GraphicsView(background="w")
@@ -412,12 +409,14 @@ class OWROCAnalysis(widget.OWWidget):
         axis = self.plot.getAxis("bottom")
         axis.setTickFont(tickfont)
         axis.setPen(pen)
-        axis.setLabel("假阳性率损失 (1-特异度)")
+        axis.setLabel("假阳性率 (1-特异度)")
+        axis.setGrid(16)
 
         axis = self.plot.getAxis("left")
         axis.setTickFont(tickfont)
         axis.setPen(pen)
         axis.setLabel("真阳性率 (灵敏度)")
+        axis.setGrid(16)
 
         self.plot.showGrid(True, True, alpha=0.1)
         self.plot.setRange(xRange=(0.0, 1.0), yRange=(0.0, 1.0), padding=0.05)
@@ -446,7 +445,6 @@ class OWROCAnalysis(widget.OWWidget):
         self.classifier_names = []
         self.selected_classifiers = []
         self.target_cb.clear()
-        self.target_index = 0
         self.colors = []
         self._curve_data = {}
         self._plot_curves = {}
@@ -471,6 +469,7 @@ class OWROCAnalysis(widget.OWWidget):
 
         class_var = results.data.domain.class_var
         self.target_cb.addItems(class_var.values)
+        self.target_index = 0
         self._set_target_prior()
 
     def _set_target_prior(self):
@@ -621,6 +620,8 @@ class OWROCAnalysis(widget.OWWidget):
         if self.roc_averaging == OWROCAnalysis.Merge:
             self._update_perf_line()
 
+        self._update_axes_ticks()
+
         warning = ""
         if not all(c.is_valid for c in hull_curves):
             if any(c.is_valid for c in hull_curves):
@@ -629,6 +630,22 @@ class OWROCAnalysis(widget.OWWidget):
                 warning = "All ROC curves are undefined"
         self.warning(warning)
 
+    def _update_axes_ticks(self):
+        def enumticks(a):
+            a = np.unique(a)
+            if len(a) > 15:
+                return None
+            return [[(x, f"{x:.2f}") for x in a[::-1]]]
+
+        data = self.curve_data(self.target_index, self.selected_classifiers[0])
+        points = data.merged.points
+
+        axis = self.plot.getAxis("bottom")
+        axis.setTicks(enumticks(points.fpr))
+
+        axis = self.plot.getAxis("left")
+        axis.setTicks(enumticks(points.tpr))
+
     def _on_mouse_moved(self, pos):
         target = self.target_index
         selected = self.selected_classifiers
@@ -802,10 +819,19 @@ def roc_curve_for_fold(res, fold, clf_idx, target):
         return np.array([]), np.array([]), np.array([])
 
     fold_probs = res.probabilities[clf_idx][fold][:, target]
-    return skl_metrics.roc_curve(
-        fold_actual, fold_probs, pos_label=target
+    drop_intermediate = len(fold_actual) > 20
+    fpr, tpr, thresholds = skl_metrics.roc_curve(
+        fold_actual, fold_probs, pos_label=target,
+        drop_intermediate=drop_intermediate
     )
 
+    # skl sets the first threshold to the highest threshold in the data + 1
+    # since we deal with probabilities, we (carefully) set it to 1
+    # Unrelated comparisons, thus pylint: disable=chained-comparison
+    if len(thresholds) > 1 and thresholds[1] <= 1:
+        thresholds[0] = 1
+    return fpr, tpr, thresholds
+
 
 def roc_curve_vertical_average(curves, samples=10):
     if not curves:
@@ -969,5 +995,19 @@ def roc_iso_performance_slope(fp_cost, fn_cost, p):
         return (fp_cost * (1. - p)) / (fn_cost * p)
 
 
+def _create_results():  # pragma: no cover
+    probs1 = [0.984, 0.907, 0.881, 0.865, 0.815, 0.741, 0.735, 0.635,
+              0.582, 0.554, 0.413, 0.317, 0.287, 0.225, 0.216, 0.183]
+    probs = np.array([[[1 - x, x] for x in probs1]])
+    preds = (probs > 0.5).astype(float)
+    return Results(
+        data=Orange.data.Table("heart_disease")[:16],
+        row_indices=np.arange(16),
+        actual=np.array(list(map(int, "1100111001001000"))),
+        probabilities=probs, predicted=preds
+    )
+
+
 if __name__ == "__main__":  # pragma: no cover
+    # WidgetPreview(OWROCAnalysis).run(_create_results())
     WidgetPreview(OWROCAnalysis).run(results_for_preview())
