@@ -4,7 +4,8 @@ from xml.sax.saxutils import escape
 import numpy as np
 import scipy.sparse as sp
 
-from AnyQt.QtCore import Qt, QRectF, pyqtSignal as Signal, QObject, QThread
+from AnyQt.QtCore import Qt, QRectF, pyqtSignal as Signal, QObject, QThread, \
+    pyqtSlot as Slot
 from AnyQt.QtGui import QTransform, QPen, QBrush, QColor, QPainter, \
     QPainterPath, QKeyEvent
 from AnyQt.QtWidgets import \
@@ -22,6 +23,7 @@ from Orange.widgets.settings import \
     DomainContextHandler, ContextSetting, Setting
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.utils.annotated_data import \
     create_annotated_table, create_groups_table, ANNOTATED_DATA_SIGNAL_NAME
 from Orange.widgets.utils.colorpalettes import \
@@ -171,17 +173,18 @@ N_ITERATIONS = 200
 
 
 class OWSOM(OWWidget):
-    name = "Self-Organizing Map"
-    description = "Computation of self-organizing map."
+    name = "自组织映射(Self-Organizing Map)"
+    description = "自组织映射的计算."
     icon = "icons/SOM.svg"
     keywords = ["SOM"]
+    category = 'unsupervised'
 
     class Inputs:
-        data = Input("Data", Table, replaces=['Data'])
+        data = Input("数据(Data)", Table, replaces=['Data'])
 
     class Outputs:
-        selected_data = Output("Selected Data", Table, default=True, replaces=['Selected Data'])
-        annotated_data = Output(ANNOTATED_DATA_SIGNAL_NAME, Table, replaces=['Data'])
+        selected_data = Output("数据(Selected Data)", Table, default=True, replaces=['Selected Data'])
+        annotated_data = Output("数据(Data)", Table, replaces=['Data'])
 
     settingsHandler = DomainContextHandler()
     auto_dimension = Setting(True)
@@ -206,7 +209,7 @@ class OWSOM(OWWidget):
     )
 
     class Warning(OWWidget.Warning):
-        ignoring_disc_variables = Msg("SOM ignores discrete variables.")
+        ignoring_disc_variables = Msg("SOM ignores categorical variables.")
         missing_colors = \
             Msg("Some data instances have undefined value of '{}'.")
         missing_values = \
@@ -229,14 +232,17 @@ class OWSOM(OWWidget):
         self.selection = None
         self.colors = self.thresholds = self.bin_labels = None
 
+        self._set_input_summary(None)
+        self._set_output_summary(None)
+
         box = gui.vBox(self.controlArea, box="SOM")
         shape = gui.comboBox(
-            box, self, "", items=("Hexagonal grid", "Square grid"))
+            box, self, "", items=("六角形", "方形网格"))
         shape.setCurrentIndex(1 - self.hexagonal)
 
         box2 = gui.indentedBox(box, 10)
         auto_dim = gui.checkBox(
-            box2, self, "auto_dimension", "Set dimensions automatically",
+            box2, self, "auto_dimension", "自动设置维度",
             callback=self.on_auto_dimension_changed)
         self.manual_box = box3 = gui.hBox(box2)
         spinargs = dict(
@@ -252,27 +258,27 @@ class OWSOM(OWWidget):
 
         initialization = gui.comboBox(
             box, self, "initialization",
-            items=("Initialize with PCA", "Random initialization",
-                   "Replicable random"))
+            items=("PCA 初始化", "随机初始化",
+                   "可复制的随机"))
 
         start = gui.button(
-            box, self, "Restart", callback=self.restart_som_pressed,
+            box, self, "重启", callback=self.restart_som_pressed,
             sizePolicy=(QSizePolicy.MinimumExpanding, QSizePolicy.Fixed))
 
         self.opt_controls = self.OptControls(
             shape, auto_dim, spin_x, spin_y, initialization, start)
 
-        box = gui.vBox(self.controlArea, "Color")
+        box = gui.vBox(self.controlArea, "颜色")
         gui.comboBox(
-            box, self, "attr_color", maximumContentsLength=15,
+            box, self, "attr_color", searchable=True,
             callback=self.on_attr_color_change,
-            model=DomainModel(placeholder="(Same color)",
+            model=DomainModel(placeholder="(相同颜色)",
                               valid_types=DomainModel.PRIMITIVE))
         gui.checkBox(
-            box, self, "pie_charts", label="Show pie charts",
+            box, self, "pie_charts", label="显示饼图",
             callback=self.on_pie_chart_change)
         gui.checkBox(
-            box, self, "size_by_instances", label="Size by number of instances",
+            box, self, "size_by_instances", label="按实例数量设置大小",
             callback=self.on_attr_size_change)
 
         gui.rubber(self.controlArea)
@@ -353,25 +359,18 @@ class OWSOM(OWWidget):
         self.set_color_bins()
         self.create_legend()
         self.recompute_dimensions()
-        self._set_input_summary(data and len(data))
+        self._set_input_summary(data)
         self.start_som()
 
-    def _set_input_summary(self, n_tot):
-        if self.data is None:
-            self.info.set_input_summary(self.info.NoInput)
-            return
+    def _set_input_summary(self, data):
+        summary = len(data) if data else self.info.NoInput
+        details = format_summary_details(data) if data else ""
+        self.info.set_input_summary(summary, details)
 
-        n = len(self.data)
-        inst = str(n)
-        nvars = f"{self.cont_x.shape[1]} numeric variables"
-        if n < n_tot:
-            inst += f" ({n_tot})"
-            details = f"{n_tot - n} out of {n_tot} instances ignored " \
-                      f"because of missing values;\n{nvars}"
-        else:
-            details = f"{n} instances; {nvars}"
-
-        self.info.set_input_summary(inst, details)
+    def _set_output_summary(self, output):
+        summary = len(output) if output else self.info.NoOutput
+        details = format_summary_details(output) if output else ""
+        self.info.set_output_summary(summary, details)
 
     def clear(self):
         self.data = self.cont_x = None
@@ -501,6 +500,7 @@ class OWSOM(OWWidget):
     def restart_som_pressed(self):
         if self._optimizer_thread is not None:
             self.stop_optimization = True
+            self._optimizer.stop_optimization = True
         else:
             self.start_som()
 
@@ -524,7 +524,7 @@ class OWSOM(OWWidget):
         c = self.opt_controls
         c.shape.setEnabled(enable)
         c.auto_dim.setEnabled(enable)
-        c.start.setText("Start" if enable else "Stop")
+        c.start.setText("开始" if enable else "停止")
 
     def update_layout(self):
         self.set_legend_pos()
@@ -637,6 +637,7 @@ class OWSOM(OWWidget):
     def _draw_colored_circles(self, sizes):
         fx, fy = self._grid_factors
         color_column = self._get_color_column()
+        qcolors = self.colors.qcolors_w_nan
         for y in range(self.size_y):
             for x in range(self.size_x - self.hexagonal * (y % 2)):
                 r = sizes[x, y]
@@ -648,10 +649,10 @@ class OWSOM(OWWidget):
                 if len(color_dist) != len(members):
                     self.Warning.missing_colors(self.attr_color.name)
                 bc = np.bincount(color_dist, minlength=len(self.colors))
-                color = self.colors[np.argmax(bc)]
+                color = qcolors[np.argmax(bc)]
                 ellipse = ColoredCircle(r / 2, color, np.max(bc) / len(members))
                 ellipse.setPos(x + (y % 2) * fx, y * fy)
-                ellipse.setToolTip(self._tooltip(self.colors, bc))
+                ellipse.setToolTip(self._tooltip(qcolors, bc))
                 self.elements.addToGroup(ellipse)
 
     def redraw_grid(self):
@@ -680,26 +681,29 @@ class OWSOM(OWWidget):
         if self.cont_x is None:
             return
 
+        som = SOM(
+            self.size_x, self.size_y,
+            hexagonal=self.hexagonal,
+            pca_init=self.initialization == 0,
+            random_seed=0 if self.initialization == 2 else None
+        )
+
         class Optimizer(QObject):
             update = Signal(float, np.ndarray, np.ndarray)
             done = Signal(SOM)
             stopped = Signal()
+            stop_optimization = False
 
-            def __init__(self, data, widget):
+            def __init__(self, data, som):
                 super().__init__()
-                self.som = SOM(
-                    widget.size_x, widget.size_y,
-                    hexagonal=widget.hexagonal,
-                    pca_init=widget.initialization == 0,
-                    random_seed=0 if widget.initialization == 2 else None)
+                self.som = som
                 self.data = data
-                self.widget = widget
 
             def callback(self, progress):
                 self.update.emit(
                     progress,
                     self.som.weights.copy(), self.som.ssum_weights.copy())
-                return not self.widget.stop_optimization
+                return not self.stop_optimization
 
             def run(self):
                 try:
@@ -710,34 +714,17 @@ class OWSOM(OWWidget):
                     self.done.emit(self.som)
                     self.stopped.emit()
 
-        def update(_progress, weights, ssum_weights):
-            progressbar.advance()
-            self._assign_instances(weights, ssum_weights)
-            self._redraw()
-
-        def done(som):
-            self.enable_controls(True)
-            progressbar.finish()
-            self._assign_instances(som.weights, som.ssum_weights)
-            self._redraw()
-            # This is the first time we know what was selected (assuming that
-            # initialization is not set to random)
-            if self.__pending_selection is not None:
-                self.on_selection_change(self.__pending_selection)
-                self.__pending_selection = None
-            self.update_output()
-
         def thread_finished():
             self._optimizer = None
             self._optimizer_thread = None
 
-        progressbar = gui.ProgressBar(self, N_ITERATIONS)
+        self.progressBarInit()
 
-        self._optimizer = Optimizer(self.cont_x, self)
+        self._optimizer = Optimizer(self.cont_x, som)
         self._optimizer_thread = QThread()
         self._optimizer_thread.setStackSize(5 * 2 ** 20)
-        self._optimizer.update.connect(update)
-        self._optimizer.done.connect(done)
+        self._optimizer.update.connect(self.__update)
+        self._optimizer.done.connect(self.__done)
         self._optimizer.stopped.connect(self._optimizer_thread.quit)
         self._optimizer.moveToThread(self._optimizer_thread)
         self._optimizer_thread.started.connect(self._optimizer.run)
@@ -745,9 +732,29 @@ class OWSOM(OWWidget):
         self.stop_optimization = False
         self._optimizer_thread.start()
 
+    @Slot(float, object, object)
+    def __update(self, _progress, weights, ssum_weights):
+        self.progressBarSet(_progress)
+        self._assign_instances(weights, ssum_weights)
+        self._redraw()
+
+    @Slot(object)
+    def __done(self, som):
+        self.enable_controls(True)
+        self.progressBarFinished()
+        self._assign_instances(som.weights, som.ssum_weights)
+        self._redraw()
+        # This is the first time we know what was selected (assuming that
+        # initialization is not set to random)
+        if self.__pending_selection is not None:
+            self.on_selection_change(self.__pending_selection)
+            self.__pending_selection = None
+        self.update_output()
+
     def stop_optimization_and_wait(self):
         if self._optimizer_thread is not None:
             self.stop_optimization = True
+            self._optimizer.stop_optimization = True
             self._optimizer_thread.quit()
             self._optimizer_thread.wait()
             self._optimizer_thread = None
@@ -806,7 +813,7 @@ class OWSOM(OWWidget):
         if self.data is None:
             self.Outputs.selected_data.send(None)
             self.Outputs.annotated_data.send(None)
-            self.info.set_output_summary(self.info.NoOutput)
+            self._set_output_summary(None)
             return
 
         indices = np.zeros(len(self.data), dtype=int)
@@ -819,10 +826,10 @@ class OWSOM(OWWidget):
         if np.any(indices):
             sel_data = create_groups_table(self.data, indices, False, "Group")
             self.Outputs.selected_data.send(sel_data)
-            self.info.set_output_summary(str(len(sel_data)))
+            self._set_output_summary(sel_data)
         else:
             self.Outputs.selected_data.send(None)
-            self.info.set_output_summary(self.info.NoOutput)
+            self._set_output_summary(None)
 
         if np.max(indices) > 1:
             annotated = create_groups_table(self.data, indices)
@@ -863,7 +870,7 @@ class OWSOM(OWWidget):
 
         items = []
         size = 8
-        for name, color in zip(names, self.colors):
+        for name, color in zip(names, self.colors.qcolors):
             item = QGraphicsItemGroup()
             item.addToGroup(
                 CanvasRectangle(None, -size / 2, -size / 2, size, size,
