@@ -8,9 +8,10 @@ from scipy.stats import norm, rayleigh, beta, gamma, pareto, expon
 from AnyQt.QtWidgets import QGraphicsRectItem
 from AnyQt.QtGui import QColor, QPen, QBrush, QPainter, QPalette, QPolygonF
 from AnyQt.QtCore import Qt, QRectF, QPointF, pyqtSignal as Signal
+from orangewidget.utils.listview import ListViewSearch
 import pyqtgraph as pg
 
-from Orange.data import Table, DiscreteVariable, ContinuousVariable
+from Orange.data import Table, DiscreteVariable, ContinuousVariable, Domain
 from Orange.preprocess.discretize import decimal_binnings, time_binnings, \
     short_time_units
 from Orange.statistics import distribution, contingency
@@ -20,6 +21,7 @@ from Orange.widgets.utils.annotated_data import \
     ANNOTATED_DATA_SIGNAL_Chinese_NAME
 from Orange.widgets.utils.itemmodels import DomainModel
 from Orange.widgets.utils.widgetpreview import WidgetPreview
+from Orange.widgets.utils.state_summary import format_summary_details
 from Orange.widgets.visualize.utils.plotutils import ElidedLabelsAxis
 from Orange.widgets.widget import Input, Output, OWWidget, Msg
 
@@ -66,7 +68,7 @@ class LegendItem(SPGLegendItem):
 
 class DistributionBarItem(pg.GraphicsObject):
     def __init__(self, x, width, padding, freqs, colors, stacked, expanded,
-                 tooltip, hidden):
+                 tooltip, desc, hidden):
         super().__init__()
         self.x = x
         self.width = width
@@ -79,6 +81,7 @@ class DistributionBarItem(pg.GraphicsObject):
         self.polygon = None
         self.hovered = False
         self._tooltip = tooltip
+        self.desc = desc
         self.hidden = False
         self.setHidden(hidden)
         self.setAcceptHoverEvents(True)
@@ -229,8 +232,8 @@ class ElidedAxisNoUnits(ElidedLabelsAxis):
                  maxTickLength=-5, showValues=True):
         self.show_unit = False
         self.tick_dict = {}
-        super().__init__(orientation, pen, linkView, parent, maxTickLength,
-                         showValues)
+        super().__init__(orientation, pen=pen, linkView=linkView, parent=parent,
+                         maxTickLength=maxTickLength, showValues=showValues)
 
     def setShowUnit(self, show_unit):
         self.show_unit = show_unit
@@ -248,7 +251,8 @@ class OWDistributions(OWWidget):
     description = "在图形中显示数据特征的值分布。"
     icon = "icons/Distribution.svg"
     priority = 120
-    keywords = []
+    keywords = ["histogram", 'fenbu', 'zhifangtu']
+    category = 'visualize'
 
     class Inputs:
         data = Input("数据(Data)", Table, doc="Set the input dataset", replaces=['Data'])
@@ -279,6 +283,7 @@ class OWDistributions(OWWidget):
     show_probs = settings.Setting(False)
     stacked_columns = settings.Setting(False)
     cumulative_distr = settings.Setting(False)
+    sort_by_freq = settings.Setting(False)
     kde_smoothing = settings.Setting(10)
 
     auto_apply = settings.Setting(True)
@@ -287,11 +292,11 @@ class OWDistributions(OWWidget):
 
     Fitters = (
         ("None", None, (), ()),
-        ("正态分布", norm, ("loc", "scale"), ("μ", "σ²")),
+        ("正态分布", norm, ("loc", "scale"), ("μ", "σ")),
         ("Beta", beta, ("a", "b", "loc", "scale"),
          ("α", "β", "-loc", "-scale")),
         ("Gamma", gamma, ("a", "loc", "scale"), ("α", "β", "-loc", "-scale")),
-        ("Rayleigh", rayleigh, ("loc", "scale"), ("-loc", "σ²")),
+        ("Rayleigh", rayleigh, ("loc", "scale"), ("-loc", "σ")),
         ("Pareto", pareto, ("b", "loc", "scale"), ("α", "-loc", "-scale")),
         ("指数分布(Exponential)", expon, ("loc", "scale"), ("-loc", "λ")),
         ("核密度(Kernel density)", AshCurve, ("a",), ("",))
@@ -313,11 +318,16 @@ class OWDistributions(OWWidget):
         self.key_operation = None
         self._user_var_bins = {}
 
-        gui.listView(
+        varview = gui.listView(
             self.controlArea, self, "var", box="变量",
             model=DomainModel(valid_types=DomainModel.PRIMITIVE,
                               separators=False),
-            callback=self._on_var_changed)
+            callback=self._on_var_changed,
+            viewType=ListViewSearch
+        )
+        gui.checkBox(
+            varview.box, self, "sort_by_freq", "按频率对类别进行排序",
+            callback=self._on_sort_by_freq, stateWhenDisabled=False)
 
         box = self.continuous_box = gui.vBox(self.controlArea, "分布")
         slider = gui.hSlider(
@@ -346,6 +356,7 @@ class OWDistributions(OWWidget):
         box = gui.vBox(self.controlArea, "列")
         gui.comboBox(
             box, self, "cvar", label="由...分割", orientation=Qt.Horizontal,
+            searchable=True,
             model=DomainModel(placeholder="(None)",
                               valid_types=(DiscreteVariable), ),
             callback=self._on_cvar_changed, contentsLength=18)
@@ -357,10 +368,13 @@ class OWDistributions(OWWidget):
             callback=self._on_show_probabilities_changed)
         gui.checkBox(
             box, self, "cumulative_distr", "Show cumulative distribution",
-            callback=self.replot)
+            callback=self._on_show_cumulative)
 
         gui.auto_apply(self.controlArea, self, commit=self.apply)
 
+        self.info.set_input_summary(self.info.NoInput)
+        self.info.set_output_summary(self.info.NoOutput)
+
         self._set_smoothing_visibility()
         self._setup_plots()
         self._setup_legend()
@@ -374,7 +388,7 @@ class OWDistributions(OWWidget):
             plot.setZValue(zvalue)
             return plot
 
-        self.plotview = DistributionWidget(background=None)
+        self.plotview = DistributionWidget()
         self.plotview.item_clicked.connect(self._on_item_clicked)
         self.plotview.blank_clicked.connect(self._on_blank_clicked)
         self.plotview.mouse_released.connect(self._on_end_selecting)
@@ -425,6 +439,9 @@ class OWDistributions(OWWidget):
         self.closeContext()
         self.var = self.cvar = None
         self.data = data
+        summary = len(data) if data else self.info.NoInput
+        details = format_summary_details(data) if data else ""
+        self.info.set_input_summary(summary, details)
         domain = self.data.domain if self.data else None
         varmodel = self.controls.var.model()
         cvarmodel = self.controls.cvar.model()
@@ -454,6 +471,14 @@ class OWDistributions(OWWidget):
         self.replot()
         self.apply()
 
+    def _on_show_cumulative(self):
+        self.replot()
+        self.apply()
+
+    def _on_sort_by_freq(self):
+        self.replot()
+        self.apply()
+
     def _on_bins_changed(self):
         self.reset_select()
         self._set_bin_width_slider_label()
@@ -569,6 +594,7 @@ class OWDistributions(OWWidget):
 
     def _update_controls_state(self):
         assert self.is_valid  # called only from replot, so assumes data is OK
+        self.controls.sort_by_freq.setDisabled(self.var.is_continuous)
         self.continuous_box.setDisabled(self.var.is_discrete)
         self.controls.show_probs.setDisabled(self.cvar is None)
         self.controls.stacked_columns.setDisabled(self.cvar is None)
@@ -589,46 +615,63 @@ class OWDistributions(OWWidget):
         self.plot.autoRange()
 
     def _add_bar(self, x, width, padding, freqs, colors, stacked, expanded,
-                 tooltip, hidden=False):
+                 tooltip, desc, hidden=False):
         item = DistributionBarItem(
             x, width, padding, freqs, colors, stacked, expanded, tooltip,
-            hidden)
+            desc, hidden)
         self.plot.addItem(item)
         self.bar_items.append(item)
 
     def _disc_plot(self):
         var = self.var
-        self.ploti.getAxis("bottom").setTicks([list(enumerate(var.values))])
-        colors = [QColor(0, 128, 255)]
         dist = distribution.get_distribution(self.data, self.var)
-        for i, freq in enumerate(dist):
+        dist = np.array(dist)  # Distribution misbehaves in further operations
+        if self.sort_by_freq:
+            order = np.argsort(dist)[::-1]
+        else:
+            order = np.arange(len(dist))
+
+        ordered_values = np.array(var.values)[order]
+        self.ploti.getAxis("bottom").setTicks([list(enumerate(ordered_values))])
+
+        colors = [QColor(0, 128, 255)]
+        for i, freq, desc in zip(count(), dist[order], ordered_values):
             tooltip = \
                 "<p style='white-space:pre;'>" \
-                f"<b>{escape(var.values[i])}</b>: {int(freq)} " \
+                f"<b>{escape(desc)}</b>: {int(freq)} " \
                 f"({100 * freq / len(self.valid_data):.2f} %) "
             self._add_bar(
                 i - 0.5, 1, 0.1, [freq], colors,
-                stacked=False, expanded=False, tooltip=tooltip)
+                stacked=False, expanded=False, tooltip=tooltip, desc=desc)
 
     def _disc_split_plot(self):
         var = self.var
-        self.ploti.getAxis("bottom").setTicks([list(enumerate(var.values))])
+        conts = contingency.get_contingency(self.data, self.cvar, self.var)
+        conts = np.array(conts)  # Contingency misbehaves in further operations
+        if self.sort_by_freq:
+            order = np.argsort(conts.sum(axis=1))[::-1]
+        else:
+            order = np.arange(len(conts))
+
+        ordered_values = np.array(var.values)[order]
+        self.ploti.getAxis("bottom").setTicks([list(enumerate(ordered_values))])
+
         gcolors = [QColor(*col) for col in self.cvar.colors]
         gvalues = self.cvar.values
-        conts = contingency.get_contingency(self.data, self.cvar, self.var)
         total = len(self.data)
-        for i, freqs in enumerate(conts):
+        for i, freqs, desc in zip(count(), conts[order], ordered_values):
             self._add_bar(
                 i - 0.5, 1, 0.1, freqs, gcolors,
                 stacked=self.stacked_columns, expanded=self.show_probs,
                 tooltip=self._split_tooltip(
-                    var.values[i], np.sum(freqs), total, gvalues, freqs))
+                    desc, np.sum(freqs), total, gvalues, freqs),
+                desc=desc)
 
     def _cont_plot(self):
         self._set_cont_ticks()
         data = self.valid_data
-        y, x = np.histogram(
-            data, bins=self.binnings[self.number_of_bins].thresholds)
+        binning = self.binnings[self.number_of_bins]
+        y, x = np.histogram(data, bins=binning.thresholds)
         total = len(data)
         colors = [QColor(0, 128, 255)]
         if self.fitted_distribution:
@@ -636,16 +679,22 @@ class OWDistributions(OWWidget):
 
         tot_freq = 0
         lasti = len(y) - 1
+        width = np.min(x[1:] - x[:-1])
+        unique = self.number_of_bins == 0 and binning.width is None
+        xoff = -width / 2 if unique else 0
         for i, (x0, x1), freq in zip(count(), zip(x, x[1:]), y):
             tot_freq += freq
+            desc = self.str_int(x0, x1, not i, i == lasti, unique)
             tooltip = \
                 "<p style='white-space:pre;'>" \
-                f"<b>{escape(self.str_int(x0, x1, not i, i == lasti))}</b>: " \
+                f"<b>{escape(desc)}</b>: " \
                 f"{freq} ({100 * freq / total:.2f} %)</p>"
+            bar_width = width if unique else x1 - x0
             self._add_bar(
-                x0, x1 - x0, 0, [tot_freq if self.cumulative_distr else freq],
+                x0 + xoff, bar_width, 0,
+                [tot_freq if self.cumulative_distr else freq],
                 colors, stacked=False, expanded=False, tooltip=tooltip,
-                hidden=self.hide_bars)
+                desc=desc, hidden=self.hide_bars)
 
         if self.fitted_distribution:
             self._plot_approximations(
@@ -655,8 +704,8 @@ class OWDistributions(OWWidget):
     def _cont_split_plot(self):
         self._set_cont_ticks()
         data = self.valid_data
-        _, bins = np.histogram(
-            data, bins=self.binnings[self.number_of_bins].thresholds)
+        binning = self.binnings[self.number_of_bins]
+        _, bins = np.histogram(data, bins=binning.thresholds)
         gvalues = self.cvar.values
         varcolors = [QColor(*col) for col in self.cvar.colors]
         if self.fitted_distribution:
@@ -678,16 +727,22 @@ class OWDistributions(OWWidget):
         tot_freqs = np.zeros(len(ys))
 
         lasti = len(ys[0]) - 1
+        width = np.min(bins[1:] - bins[:-1])
+        unique = self.number_of_bins == 0 and binning.width is None
+        xoff = -width / 2 if unique else 0
         for i, x0, x1, freqs in zip(count(), bins, bins[1:], zip(*ys)):
             tot_freqs += freqs
             plotfreqs = tot_freqs.copy() if self.cumulative_distr else freqs
+            desc = self.str_int(x0, x1, not i, i == lasti, unique)
+            bar_width = width if unique else x1 - x0
             self._add_bar(
-                x0, x1 - x0, 0 if self.stacked_columns else 0.1, plotfreqs,
+                x0 + xoff, bar_width, 0 if self.stacked_columns else 0.1,
+                plotfreqs,
                 gcolors, stacked=self.stacked_columns, expanded=self.show_probs,
                 hidden=self.hide_bars,
                 tooltip=self._split_tooltip(
-                    self.str_int(x0, x1, not i, i == lasti),
-                    np.sum(plotfreqs), total, gvalues, plotfreqs))
+                    desc, np.sum(plotfreqs), total, gvalues, plotfreqs),
+                desc=desc)
 
         if fitters:
             self._plot_approximations(bins[0], bins[-1], fitters, varcolors,
@@ -856,12 +911,12 @@ class OWDistributions(OWWidget):
             return 0
         return 10 ** -var.number_of_decimals
 
-    def str_int(self, x0, x1, first, last):
+    def str_int(self, x0, x1, first, last, unique=False):
         var = self.var
         sx0, sx1 = var.repr_val(x0), var.repr_val(x1)
         if self.cumulative_distr:
             return f"{var.name} < {sx1}"
-        elif first and last:
+        elif first and last or unique:
             return f"{var.name} = {sx0}"
         elif first:
             return f"{var.name} < {sx1}"
@@ -1066,15 +1121,21 @@ class OWDistributions(OWWidget):
                 group_indices, values = self._get_output_indices_disc()
             else:
                 group_indices, values = self._get_output_indices_cont()
-                hist_indices, hist_values = self._get_histogram_indices()
-                histogram_data = create_groups_table(
-                    data, hist_indices, values=hist_values)
             selected = np.nonzero(group_indices)[0]
             if selected.size:
                 selected_data = create_groups_table(
                     data, group_indices,
                     include_unselected=False, values=values)
-                annotated_data = create_annotated_table(data, selected)
+            annotated_data = create_annotated_table(data, selected)
+            if self.var.is_continuous:  # annotate with bins
+                hist_indices, hist_values = self._get_histogram_indices()
+                annotated_data = create_groups_table(
+                    annotated_data, hist_indices, var_name="Bin", values=hist_values)
+            histogram_data = self._get_histogram_table()
+
+        summary = len(selected_data) if selected_data else self.info.NoOutput
+        details = format_summary_details(selected_data) if selected_data else ""
+        self.info.set_output_summary(summary, details)
 
         self.Outputs.selected_data.send(selected_data)
         self.Outputs.annotated_data.send(annotated_data)
@@ -1105,6 +1166,21 @@ class OWDistributions(OWWidget):
                 self.str_int(x0, x1, not bar_idx, self._is_last_bar(bar_idx)))
         return group_indices, values
 
+    def _get_histogram_table(self):
+        var_bin = DiscreteVariable("Bin", [bar.desc for bar in self.bar_items])
+        var_freq = ContinuousVariable("Count")
+        X = []
+        if self.cvar:
+            domain = Domain([var_bin, self.cvar, var_freq])
+            for i, bar in enumerate(self.bar_items):
+                for j, freq in enumerate(bar.freqs):
+                    X.append([i, j, freq])
+        else:
+            domain = Domain([var_bin, var_freq])
+            for i, bar in enumerate(self.bar_items):
+                X.append([i, bar.freqs[0]])
+        return Table.from_numpy(domain, X)
+
     def _get_histogram_indices(self):
         group_indices = np.zeros(len(self.data), dtype=np.int32)
         col = self.data.get_column_view(self.var)[0].astype(float)
