@@ -177,9 +177,9 @@ class LinearProjectionVizRank(VizRankDialog, OWComponent):
 
     def _n_attrs_changed(self):
         if self.n_attrs != self.last_run_n_attrs or self.saved_state is None:
-            self.button.setText("Start")
+            self.button.setText("开始")
         else:
-            self.button.setText("Continue")
+            self.button.setText("继续")
         self.button.setEnabled(self.check_preconditions())
 
 
@@ -225,6 +225,7 @@ class OWLinProjGraph(OWGraphWithAnchors):
                 anchor._label.setToolTip(f"<b>{label}</b>")
                 label = label[:MAX_LABEL_LEN - 3] + "..." if len(label) > MAX_LABEL_LEN else label
                 anchor.setText(label)
+                anchor.setFont(self.parameter_setter.anchor_font)
 
                 visible = self.always_show_axes or np.linalg.norm(point) > r
                 anchor.setVisible(visible)
@@ -236,6 +237,7 @@ class OWLinProjGraph(OWGraphWithAnchors):
                 anchor.setLine(QLineF(0, 0, *point))
                 visible = self.always_show_axes or np.linalg.norm(point) > r
                 anchor.setVisible(visible)
+                anchor.setFont(self.parameter_setter.anchor_font)
 
     def update_circle(self):
         super().update_circle()
@@ -264,7 +266,8 @@ class OWLinearProjection(OWAnchorProjectionWidget):
     description = "数据在二维平面上的多轴投影。"
     icon = "icons/LinearProjection.svg"
     priority = 240
-    keywords = []
+    keywords = ['xianxingtouying', 'touying']
+    category = 'visualize'
 
     Projection_name = {Placement.Circular: "圆形布局",
                        Placement.LDA: "线性判别分析(Linear Discriminant Analysis)",
@@ -324,6 +327,10 @@ class OWLinearProjection(OWAnchorProjectionWidget):
     def effective_variables(self):
         return self.selected_vars
 
+    @property
+    def effective_data(self):
+        return self.data.transform(Domain(self.effective_variables))
+
     def __vizrank_set_attrs(self, attrs):
         if not attrs:
             return
@@ -539,5 +546,6 @@ class CircularPlacement(LinearProjector):
 
 
 if __name__ == "__main__":  # pragma: no cover
-    data = Table("iris")
-    WidgetPreview(OWLinearProjection).run(set_data=data, set_subset_data=data[::10])
+    iris = Table("iris")
+    WidgetPreview(OWLinearProjection).run(set_data=iris,
+                                          set_subset_data=iris[::10])
